@@ -177,6 +177,15 @@ typedef struct _GstBufferClass GstBufferClass;
  * Tests if the end offset is known.
  */
 #define GST_BUFFER_OFFSET_END_IS_VALID(buffer)	(GST_BUFFER_OFFSET_END (buffer) != GST_BUFFER_OFFSET_NONE)
+/**
+ * GST_BUFFER_IS_DISCONT:
+ * @buffer: a #GstBuffer
+ *
+ * Tests if the buffer marks a discontinuity in the stream.
+ *
+ * Since: 0.10.9
+ */
+#define GST_BUFFER_IS_DISCONT(buffer)	(GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT))
 
 /**
  * GstBufferFlag:
@@ -258,10 +267,12 @@ struct _GstBufferClass {
   GstMiniObjectClass    mini_object_class;
 };
 
+GType       gst_buffer_get_type (void);
+
 /* allocation */
-GType		gst_buffer_get_type		(void);
-GstBuffer*	gst_buffer_new			(void);
-GstBuffer*	gst_buffer_new_and_alloc	(guint size);
+GstBuffer * gst_buffer_new               (void);
+GstBuffer * gst_buffer_new_and_alloc     (guint size);
+GstBuffer * gst_buffer_try_new_and_alloc (guint size);
 
 /**
  * gst_buffer_set_data:
@@ -307,7 +318,7 @@ gst_buffer_ref (GstBuffer * buf)
 {
   /* not using a macro here because gcc-4.1 will complain
    * if the return value isn't used (because of the cast) */
-  return (GstBuffer *) gst_mini_object_ref (GST_MINI_OBJECT (buf));
+  return (GstBuffer *) gst_mini_object_ref (GST_MINI_OBJECT_CAST (buf));
 }
 
 /**
@@ -318,7 +329,7 @@ gst_buffer_ref (GstBuffer * buf)
  * will be freed. If GST_BUFFER_MALLOCDATA() is non-NULL, this pointer will
  * also be freed at this time.
  */
-#define		gst_buffer_unref(buf)		gst_mini_object_unref (GST_MINI_OBJECT (buf))
+#define		gst_buffer_unref(buf)		gst_mini_object_unref (GST_MINI_OBJECT_CAST (buf))
 
 /* copy buffer */
 /**
@@ -328,7 +339,40 @@ gst_buffer_ref (GstBuffer * buf)
  * Create a copy of the given buffer. This will also make a newly allocated
  * copy of the data the source buffer contains.
  */
-#define		gst_buffer_copy(buf)		GST_BUFFER_CAST (gst_mini_object_copy (GST_MINI_OBJECT (buf)))
+#define		gst_buffer_copy(buf)		GST_BUFFER_CAST (gst_mini_object_copy (GST_MINI_OBJECT_CAST (buf)))
+
+/**
+ * GstBufferCopyFlags:
+ * @GST_BUFFER_COPY_FLAGS: flag indicating that buffer flags should be copied
+ * @GST_BUFFER_COPY_TIMESTAMPS: flag indicating that buffer timestamp, duration,
+ * offset and offset_end should be copied
+ * @GST_BUFFER_COPY_CAPS: flag indicating that buffer caps should be copied
+ *
+ * A set of flags that can be provided to the gst_buffer_copy_metadata()
+ * function to specify which metadata fields should be copied.
+ *
+ * Since: 0.10.13
+ */
+typedef enum {
+  GST_BUFFER_COPY_FLAGS      = (1 << 0),
+  GST_BUFFER_COPY_TIMESTAMPS = (1 << 1),
+  GST_BUFFER_COPY_CAPS       = (1 << 2),
+} GstBufferCopyFlags;
+
+/**
+ * GST_BUFFER_COPY_ALL:
+ *
+ * Combination of all possible fields that can be copied with
+ * gst_buffer_copy_metadata().
+ *
+ * Since: 0.10.13
+ */
+#define GST_BUFFER_COPY_ALL (GST_BUFFER_COPY_FLAGS | GST_BUFFER_COPY_TIMESTAMPS | GST_BUFFER_COPY_CAPS)
+
+/* copies metadata into newly allocated buffer */
+void            gst_buffer_copy_metadata        (GstBuffer *dest, const GstBuffer *src,
+                                                 GstBufferCopyFlags flags);
+
 /**
  * gst_buffer_is_writable:
  * @buf: a #GstBuffer
@@ -338,7 +382,7 @@ gst_buffer_ref (GstBuffer * buf)
  * writable, but it is only safe to change it when there is only one owner
  * of the buffer - ie, the refcount is 1. 
  */
-#define		gst_buffer_is_writable(buf)	gst_mini_object_is_writable (GST_MINI_OBJECT (buf))
+#define		gst_buffer_is_writable(buf)	gst_mini_object_is_writable (GST_MINI_OBJECT_CAST (buf))
 /**
  * gst_buffer_make_writable:
  * @buf: a #GstBuffer
@@ -347,7 +391,7 @@ gst_buffer_ref (GstBuffer * buf)
  * already writable, this will simply return the same buffer. A copy will 
  * otherwise be made using gst_buffer_copy().
  */
-#define		gst_buffer_make_writable(buf)   GST_BUFFER_CAST (gst_mini_object_make_writable (GST_MINI_OBJECT (buf)))
+#define		gst_buffer_make_writable(buf)   GST_BUFFER_CAST (gst_mini_object_make_writable (GST_MINI_OBJECT_CAST (buf)))
 
 /* Ensure that the metadata of the buffer is writable, even if the buffer data
  * isn't */
@@ -367,7 +411,12 @@ GstBuffer*      gst_buffer_make_metadata_writable (GstBuffer *buf);
  *
  * Either @nbuf or the #GstBuffer pointed to by @obuf may be NULL.
  */
-#define		gst_buffer_replace(obuf,nbuf)	gst_mini_object_replace ((GstMiniObject **)(obuf), GST_MINI_OBJECT (nbuf))
+#define		gst_buffer_replace(obuf,nbuf) \
+G_STMT_START {						                      \
+  GstBuffer **___obufaddr = (GstBuffer **)(obuf);         \
+  gst_mini_object_replace ((GstMiniObject **)___obufaddr, \
+      GST_MINI_OBJECT_CAST (nbuf));                       \
+} G_STMT_END
 
 GstCaps*	gst_buffer_get_caps		(GstBuffer *buffer);
 void		gst_buffer_set_caps		(GstBuffer *buffer, GstCaps *caps);
@@ -384,28 +433,26 @@ GstBuffer*	gst_buffer_span			(GstBuffer *buf1, guint32 offset, GstBuffer *buf2,
  * @v: a #GstValue to receive the data
  * @b: a #GstBuffer to assign to the GstValue
  *
- * Sets @b as the value of @v, correclty incrementing the refcount of
- * the buffer.
+ * Sets @b as the value of @v.  Caller retains reference to buffer.
  */
-#define		gst_value_set_buffer(v,b)	gst_value_set_mini_object(v, GST_MINI_OBJECT(b))
+#define		gst_value_set_buffer(v,b)	gst_value_set_mini_object(v, GST_MINI_OBJECT_CAST(b))
 /**
  * gst_value_take_buffer:
  * @v: a #GstValue to receive the data
  * @b: a #GstBuffer to assign to the GstValue
  *
- * Sets @b as the value of @v, this function lets the GstValue
- * take ownership of the buffer.
+ * Sets @b as the value of @v.  Caller gives away reference to buffer.
  */
-#define		gst_value_take_buffer(v,b)	gst_value_take_mini_object(v, GST_MINI_OBJECT(b))
+#define		gst_value_take_buffer(v,b)	gst_value_take_mini_object(v, GST_MINI_OBJECT_CAST(b))
 /**
  * gst_value_get_buffer:
  * @v: a #GstValue to qeury
  *
- * Receives a #GstBuffer as the value of @v. This function does not
- * increase the refcount of the returned buffer so the buffer remains
- * valid as long as you own a refcount to the GstValue.
+ * Receives a #GstBuffer as the value of @v. Does not return a reference to
+ * the buffer, so the pointer is only valid for as long as the caller owns
+ * a reference to @v.
  */
-#define		gst_value_get_buffer(v)		GST_BUFFER (gst_value_get_mini_object(v))
+#define		gst_value_get_buffer(v)		GST_BUFFER_CAST (gst_value_get_mini_object(v))
 
 /* --- protected --- */
 void		_gst_buffer_initialize		(void);
