@@ -50,7 +50,7 @@ struct _GstRegistry {
   GstObject 	 object;
 
   /*< private >*/
-  GList		*plugins;
+  GList *plugins;
   GList *features;
 
   GList 	*paths;
@@ -58,8 +58,11 @@ struct _GstRegistry {
   /* FIXME move these elsewhere */
   int            cache_file;
 
+  /* hash to speedup _lookup_feature_locked() */
+  GHashTable *feature_hash;
+
   /*< private >*/
-  gpointer _gst_reserved[GST_PADDING];
+  gpointer _gst_reserved[GST_PADDING-1];
 };
 
 struct _GstRegistryClass {
@@ -106,8 +109,13 @@ GstPluginFeature*	gst_registry_find_feature	(GstRegistry *registry, const gchar
 GstPlugin * 		gst_registry_lookup 		(GstRegistry *registry, const char *filename);
 GstPluginFeature * 	gst_registry_lookup_feature 	(GstRegistry *registry, const char *name);
 
+#ifdef USE_BINARY_REGISTRY
+gboolean 		gst_registry_binary_read_cache 	(GstRegistry * registry, const char *location);
+gboolean 		gst_registry_binary_write_cache	(GstRegistry * registry, const char *location);
+#else 
 gboolean 		gst_registry_xml_read_cache 	(GstRegistry * registry, const char *location);
 gboolean 		gst_registry_xml_write_cache 	(GstRegistry * registry, const char *location);
+#endif
 
 /* convinience defines for the default registry */
 
