@@ -62,9 +62,9 @@
  *
  * For elements that perform seeks, the current segment should be updated with the
  * gst_segment_set_seek() and the values from the seek event. This method will update
- * all the segment fields. The last_pos field will contain the new playback position.
+ * all the segment fields. The last_stop field will contain the new playback position.
  * If the cur_type was different from GST_SEEK_TYPE_NONE, playback continues from
- * the last_pos position, possibly with updated flags or rate.
+ * the last_stop position, possibly with updated flags or rate.
  *
  * For elements that want to use #GstSegment to track the playback region, use
  * gst_segment_set_newsegment() to update the segment fields with the information from
@@ -80,7 +80,7 @@
  * gst_segment_to_stream_time() can be used to convert a timestamp and the segment
  * info to stream time (which is always between 0 and the duration of the stream).
  *
- * Last reviewed on 2006-05-03 (0.10.6)
+ * Last reviewed on 2007-05-17 (0.10.13)
  */
 
 static GstSegment *
@@ -230,7 +230,7 @@ gst_segment_set_last_stop (GstSegment * segment, GstFormat format,
  * @start: the seek start value
  * @stop_type: the seek method
  * @stop: the seek stop value
- * @update: boolean holding whether start or stop were updated.
+ * @update: boolean holding whether last_stop was updated.
  *
  * Update the segment structure with the field values of a seek event (see
  * gst_event_new_seek()).
@@ -255,6 +255,10 @@ gst_segment_set_last_stop (GstSegment * segment, GstFormat format,
  * The applied rate of the segment will be set to 1.0 by default.
  * If the caller can apply a rate change, it should update @segment
  * rate and applied_rate after calling this function.
+ *
+ * @update will be set to TRUE if a seek should be performed to the segment 
+ * last_stop field. This field can be FALSE if, for example, only the @rate
+ * has been changed but not the playback position.
  */
 void
 gst_segment_set_seek (GstSegment * segment, gdouble rate,
@@ -263,6 +267,7 @@ gst_segment_set_seek (GstSegment * segment, gdouble rate,
     GstSeekType stop_type, gint64 stop, gboolean * update)
 {
   gboolean update_stop, update_start;
+  gint64 last_stop;
 
   g_return_if_fail (rate != 0.0);
   g_return_if_fail (segment != NULL);
@@ -272,7 +277,7 @@ gst_segment_set_seek (GstSegment * segment, gdouble rate,
   else
     g_return_if_fail (segment->format == format);
 
-  update_stop = update_start = TRUE;
+  update_start = update_stop = TRUE;
 
   /* start is never invalid */
   switch (start_type) {
@@ -349,24 +354,30 @@ gst_segment_set_seek (GstSegment * segment, gdouble rate,
   segment->applied_rate = 1.0;
   segment->flags = flags;
   segment->start = start;
+  last_stop = segment->last_stop;
   if (update_start && rate > 0.0) {
-    segment->last_stop = start;
+    last_stop = start;
   }
   if (update_stop && rate < 0.0) {
     if (stop != -1)
-      segment->last_stop = stop;
+      last_stop = stop;
     else {
       if (segment->duration != -1)
-        segment->last_stop = segment->duration;
+        last_stop = segment->duration;
       else
-        segment->last_stop = 0;
+        last_stop = 0;
     }
   }
+  /* set update arg to reflect update of last_stop */
+  if (update)
+    *update = last_stop != segment->last_stop;
+
+  /* update new position */
+  if (last_stop != segment->last_stop)
+    segment->last_stop = last_stop;
+
   segment->time = start;
   segment->stop = stop;
-
-  if (update)
-    *update = update_start || update_stop;
 }
 
 /**
@@ -635,6 +646,9 @@ gst_segment_to_running_time (GstSegment * segment, GstFormat format,
  * updated. If @clip_start or @clip_stop are different from @start or @stop
  * respectively, the region fell partially in the segment.
  *
+ * Note that when @stop is -1, @clip_stop will be set to the end of the
+ * segment. Depending on the use case, this may or may not be what you want.
+ *
  * Returns: TRUE if the given @start and @stop times fall partially or 
  *     completely in @segment, FALSE if the values are completely outside 
  *     of the segment.
