@@ -36,7 +36,10 @@
  * <para>
  * If you just want to retrieve metadata in your application then all you
  * need to do is watch for tag messages on your pipeline's bus. This
- * interface is only for setting metadata, not for extracting it.
+ * interface is only for setting metadata, not for extracting it. To set tags
+ * from the application, find tagsetter elements and set tags using e.g.
+ * gst_tag_setter_merge_tags() or gst_tag_setter_add_tags(). The application
+ * should do that before the element goes to %GST_STATE_PAUSED.
  * </para>
  * <para>
  * Elements implementing the #GstTagSetter interface often have to merge
@@ -98,7 +101,7 @@ gst_tag_setter_get_type (void)
 {
   static GType tag_setter_type = 0;
 
-  if (!tag_setter_type) {
+  if (G_UNLIKELY (tag_setter_type == 0)) {
     static const GTypeInfo tag_setter_info = {
       sizeof (GstTagSetterIFace),       /* class_size */
       NULL,                     /* base_init */
@@ -167,12 +170,13 @@ gst_tag_setter_merge_tags (GstTagSetter * setter, const GstTagList * list,
 
   g_return_if_fail (GST_IS_TAG_SETTER (setter));
   g_return_if_fail (GST_TAG_MODE_IS_VALID (mode));
+  g_return_if_fail (GST_IS_TAG_LIST (list));
 
   data = gst_tag_setter_get_data (setter);
   if (!data->list) {
     data->list = gst_tag_list_copy (list);
   } else {
-    gst_tag_list_merge (data->list, list, mode);
+    gst_tag_list_insert (data->list, list, mode);
   }
 }
 
