@@ -2,7 +2,7 @@
  * Copyright (C) 1999,2000 Erik Walthinsen <omega@cse.ogi.edu>
  *                    2000 Wim Taymans <wtay@chello.be>
  *
- * gsttrace.c: Tracing functions (depracated)
+ * gsttrace.c: Tracing functions (deprecated)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -20,6 +20,30 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:gsttrace
+ * @short_description: Tracing functionality
+ *
+ * Traces allows to track object allocation. They provide a instance counter per
+ * #GType. The counter is incremented for each object allocated and decremented
+ * it when it's freed.
+ *
+ * <example>
+ * <title>Tracing object instances</title>
+ *   <programlisting>
+ *     // trace un-freed object instances
+ *     gst_alloc_trace_set_flags_all (GST_ALLOC_TRACE_LIVE);
+ *     if (!gst_alloc_trace_available ()) {
+ *       g_warning ("Trace not available (recompile with trace enabled).");
+ *     }
+ *     gst_alloc_trace_print_live ();
+ *     // do something here
+ *     gst_alloc_trace_print_live ();
+ *   </programlisting>
+ * </example>
+ *
+ * Last reviewed on 2005-11-21 (0.9.5)
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -54,6 +78,13 @@ read_tsc (gint64 * dst)
 #endif
 }
 
+/**
+ * gst_trace_read_tsc:
+ * @dst: pointer to hold the result.
+ *
+ * Read a platform independent timer value that can be used in
+ * benchmarks.
+ */
 void
 gst_trace_read_tsc (gint64 * dst)
 {
@@ -63,6 +94,16 @@ gst_trace_read_tsc (gint64 * dst)
 GstTrace *_gst_trace_default = NULL;
 gint _gst_trace_on = 1;
 
+/**
+ * gst_trace_new:
+ * @filename: a filename
+ * @size: the max size of the file
+ *
+ * Create a ringbuffer of @size in the file with @filename to 
+ * store trace results in.
+ *
+ * Returns: a new #GstTrace.
+ */
 GstTrace *
 gst_trace_new (gchar * filename, gint size)
 {
@@ -70,7 +111,7 @@ gst_trace_new (gchar * filename, gint size)
 
   g_return_val_if_fail (trace != NULL, NULL);
   trace->filename = g_strdup (filename);
-  GST_DEBUG ("opening '%s'\n", trace->filename);
+  GST_DEBUG ("opening '%s'", trace->filename);
 #ifndef S_IWUSR
 #define S_IWUSR S_IWRITE
 #endif
@@ -89,6 +130,12 @@ gst_trace_new (gchar * filename, gint size)
   return trace;
 }
 
+/**
+ * gst_trace_destroy:
+ * @trace: the #GstTrace to destroy
+ *
+ * Flush an close the previously allocated @trace.
+ */
 void
 gst_trace_destroy (GstTrace * trace)
 {
@@ -102,6 +149,14 @@ gst_trace_destroy (GstTrace * trace)
   g_free (trace);
 }
 
+/**
+ * gst_trace_flush:
+ * @trace: the #GstTrace to flush.
+ *
+ * Flush any pending trace entries in @trace to the trace file.
+ * @trace can be NULL in which case the default #GstTrace will be
+ * flushed.
+ */
 void
 gst_trace_flush (GstTrace * trace)
 {
@@ -111,10 +166,20 @@ gst_trace_flush (GstTrace * trace)
       return;
   }
 
-  write (trace->fd, trace->buf, trace->bufoffset * sizeof (GstTraceEntry));
+  g_return_if_fail (write (trace->fd, trace->buf,
+          trace->bufoffset * sizeof (GstTraceEntry)) != -1);
   trace->bufoffset = 0;
 }
 
+/**
+ * gst_trace_text_flush:
+ * @trace: the #GstTrace to flush.
+ *
+ * Flush any pending trace entries in @trace to the trace file, 
+ * formatted as a text line with timestamp and sequence numbers.
+ * @trace can be NULL in which case the default #GstTrace will be
+ * flushed.
+ */
 void
 gst_trace_text_flush (GstTrace * trace)
 {
@@ -133,12 +198,18 @@ gst_trace_text_flush (GstTrace * trace)
     g_snprintf (str, STRSIZE, "%20" G_GINT64_FORMAT " %10d %10d %s\n",
         trace->buf[i].timestamp,
         trace->buf[i].sequence, trace->buf[i].data, trace->buf[i].message);
-    write (trace->fd, str, strlen (str));
+    g_return_if_fail (write (trace->fd, str, strlen (str)) != -1);
   }
   trace->bufoffset = 0;
 #undef STRSIZE
 }
 
+/**
+ * gst_trace_set_default:
+ * @trace: the #GstTrace to set as the default.
+ *
+ * Set the default #GstTrace to @trace.
+ */
 void
 gst_trace_set_default (GstTrace * trace)
 {
@@ -177,7 +248,7 @@ static GList *_gst_alloc_tracers = NULL;
 /**
  * gst_alloc_trace_available:
  *
- * Check if alloc tracing was commiled into the core
+ * Check if alloc tracing was compiled into the core
  *
  * Returns: TRUE if the core was compiled with alloc
  * tracing enabled.
@@ -235,7 +306,9 @@ gst_alloc_trace_list (void)
 /**
  * gst_alloc_trace_live_all:
  *
- * Returns the total number of live registered alloc trace objects.
+ * Get the total number of live registered alloc trace objects.
+ *
+ * Returns: the total number of live registered alloc trace objects.
  */
 int
 gst_alloc_trace_live_all (void)
@@ -254,15 +327,34 @@ gst_alloc_trace_live_all (void)
   return num;
 }
 
+static gint
+compare_func (GstAllocTrace * a, GstAllocTrace * b)
+{
+  return strcmp (a->name, b->name);
+}
+
+static GList *
+gst_alloc_trace_list_sorted (void)
+{
+  GList *ret;
+
+  ret = g_list_sort (g_list_copy (_gst_alloc_tracers),
+      (GCompareFunc) compare_func);
+
+  return ret;
+}
+
 /**
  * gst_alloc_trace_print_all:
  *
- * Print the status of all registered alloc trace objectes.
+ * Print the status of all registered alloc trace objects.
  */
 void
 gst_alloc_trace_print_all (void)
 {
-  GList *walk = _gst_alloc_tracers;
+  GList *orig, *walk;
+
+  orig = walk = gst_alloc_trace_list_sorted ();
 
   while (walk) {
     GstAllocTrace *trace = (GstAllocTrace *) walk->data;
@@ -271,6 +363,33 @@ gst_alloc_trace_print_all (void)
 
     walk = g_list_next (walk);
   }
+
+  g_list_free (orig);
+}
+
+/**
+ * gst_alloc_trace_print_live:
+ *
+ * Print the status of all registered alloc trace objects, ignoring those
+ * without live objects.
+ */
+void
+gst_alloc_trace_print_live (void)
+{
+  GList *orig, *walk;
+
+  orig = walk = gst_alloc_trace_list_sorted ();
+
+  while (walk) {
+    GstAllocTrace *trace = (GstAllocTrace *) walk->data;
+
+    if (trace->live)
+      gst_alloc_trace_print (trace);
+
+    walk = g_list_next (walk);
+  }
+
+  g_list_free (orig);
 }
 
 /**
@@ -288,7 +407,7 @@ gst_alloc_trace_set_flags_all (GstAllocTraceFlags flags)
   while (walk) {
     GstAllocTrace *trace = (GstAllocTrace *) walk->data;
 
-    GST_DEBUG ("set flags on %p\n", trace);
+    GST_DEBUG ("setting flags %d on %p", (gint) flags, trace);
     gst_alloc_trace_set_flags (trace, flags);
 
     walk = g_list_next (walk);
@@ -336,33 +455,26 @@ gst_alloc_trace_print (const GstAllocTrace * trace)
 
   g_return_if_fail (trace != NULL);
 
-  g_print ("%s (%p): flags %d", trace->name, trace, trace->flags);
-
   if (trace->flags & GST_ALLOC_TRACE_LIVE) {
-    g_print (", live %d", trace->live);
+    g_print ("%-22.22s : %d\n", trace->name, trace->live);
+  } else {
+    g_print ("%-22.22s : (no live count)\n", trace->name);
   }
+
   if (trace->flags & GST_ALLOC_TRACE_MEM_LIVE) {
     mem_live = trace->mem_live;
 
-    if (!mem_live) {
-      g_print (", no live memory");
-    } else {
-      g_print (", dumping live memory: ");
-
-      while (mem_live) {
-        g_print ("%p ", mem_live->data);
-        mem_live = g_slist_next (mem_live);
-      }
-      g_print ("\ntotal %d", g_slist_length (trace->mem_live));
+    while (mem_live) {
+      g_print ("%-22.22s : %p\n", "", mem_live->data);
+      mem_live = mem_live->next;
     }
   }
-  g_print ("\n");
 }
 
 /**
  * gst_alloc_trace_set_flags:
- * @trace: the GstAllocTrace 
- * @flags: flags to set 
+ * @trace: the GstAllocTrace
+ * @flags: flags to set
  *
  * Enable the given features on the given GstAllocTrace object.
  */
