@@ -71,7 +71,7 @@ typedef struct _GstURIHandlerInterface GstURIHandlerInterface;
 /**
  * GstURIHandlerInterface:
  * @parent: The parent interface type
- * @get_type: Method to tell wether the element handles source or sink URI.
+ * @get_type: Method to tell whether the element handles source or sink URI.
  * @get_protocols: Method to return the list of protocols handled by the element.
  * @get_uri: Method to return the URI currently handled by the element.
  * @set_uri: Method to set a new URI.
@@ -113,8 +113,12 @@ struct _GstURIHandlerInterface {
 /* general URI functions */
 
 gboolean	gst_uri_protocol_is_valid	(const gchar * protocol);
+gboolean	gst_uri_protocol_is_supported	(const GstURIType type,
+						 const gchar *protocol);
 gboolean	gst_uri_is_valid		(const gchar * uri);
 gchar *		gst_uri_get_protocol		(const gchar * uri);
+gboolean        gst_uri_has_protocol            (const gchar * uri,
+                                                 const gchar * protocol);
 gchar *		gst_uri_get_location		(const gchar * uri);
 gchar *		gst_uri_construct		(const gchar * protocol,
 						 const gchar * location);
