@@ -1970,7 +1970,8 @@ gst_element_query_convert (GstElement * element, GstFormat src_format,
  * gst_element_seek_simple
  * @element: a #GstElement to seek on
  * @format: a #GstFormat to execute the seek in, such as #GST_FORMAT_TIME
- * @seek_flags: seek options
+ * @seek_flags: seek options; playback applications will usually want to use
+ *            GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT here
  * @seek_pos: position to seek to (relative to the start); if you are doing
  *            a seek in #GST_FORMAT_TIME this value is in nanoseconds -
  *            multiply with #GST_SECOND to convert seconds to nanoseconds or
@@ -2424,11 +2425,13 @@ gst_buffer_join (GstBuffer * buf1, GstBuffer * buf2)
  * Deprecated: use gst_buffer_copy_metadata() instead, it provides more
  * control.
  */
+#ifndef GST_REMOVE_DEPRECATED
 void
 gst_buffer_stamp (GstBuffer * dest, const GstBuffer * src)
 {
   gst_buffer_copy_metadata (dest, src, GST_BUFFER_COPY_TIMESTAMPS);
 }
+#endif
 
 static gboolean
 intersect_caps_func (GstPad * pad, GValue * ret, GstPad * orig)
@@ -3254,3 +3257,72 @@ gst_parse_bin_from_description (const gchar * bin_description,
   return NULL;
 #endif
 }
+
+/**
+ * gst_type_register_static_full:
+ * @parent_type: The GType of the parent type the newly registered type will 
+ *   derive from
+ * @type_name: NULL-terminated string used as the name of the new type
+ * @class_size: Size of the class structure.
+ * @base_init: Location of the base initialization function (optional).
+ * @base_finalize: Location of the base finalization function (optional).
+ * @class_init: Location of the class initialization function for class types 
+ *   Location of the default vtable inititalization function for interface 
+ *   types. (optional)
+ * @class_finalize: Location of the class finalization function for class types.
+ *   Location of the default vtable finalization function for interface types. 
+ *   (optional)
+ * @class_data: User-supplied data passed to the class init/finalize functions.
+ * @instance_size: Size of the instance (object) structure (required for 
+ *   instantiatable types only).
+ * @n_preallocs: The number of pre-allocated (cached) instances to reserve 
+ *   memory for (0 indicates no caching). Ignored on recent GLib's.
+ * @instance_init: Location of the instance initialization function (optional, 
+ *   for instantiatable types only).
+ * @value_table: A GTypeValueTable function table for generic handling of 
+ *   GValues of this type (usually only useful for fundamental types). 
+ * @flags: #GTypeFlags for this GType. E.g: G_TYPE_FLAG_ABSTRACT
+ *
+ * Helper function which constructs a #GTypeInfo structure and registers a 
+ * GType, but which generates less linker overhead than a static const 
+ * #GTypeInfo structure. For further details of the parameters, please see
+ * #GTypeInfo in the GLib documentation.
+ *
+ * Registers type_name as the name of a new static type derived from 
+ * parent_type. The value of flags determines the nature (e.g. abstract or 
+ * not) of the type. It works by filling a GTypeInfo struct and calling 
+ * g_type_info_register_static().
+ *
+ * Returns: A #GType for the newly-registered type.
+ *
+ * Since: 0.10.14
+ */
+GType
+gst_type_register_static_full (GType parent_type,
+    const gchar * type_name,
+    guint class_size,
+    GBaseInitFunc base_init,
+    GBaseFinalizeFunc base_finalize,
+    GClassInitFunc class_init,
+    GClassFinalizeFunc class_finalize,
+    gconstpointer class_data,
+    guint instance_size,
+    guint16 n_preallocs,
+    GInstanceInitFunc instance_init,
+    const GTypeValueTable * value_table, GTypeFlags flags)
+{
+  GTypeInfo info;
+
+  info.class_size = class_size;
+  info.base_init = base_init;
+  info.base_finalize = base_finalize;
+  info.class_init = class_init;
+  info.class_finalize = class_finalize;
+  info.class_data = class_data;
+  info.instance_size = instance_size;
+  info.n_preallocs = n_preallocs;
+  info.instance_init = instance_init;
+  info.value_table = value_table;
+
+  return g_type_register_static (parent_type, type_name, &info, flags);
+}
