@@ -126,7 +126,7 @@
  * The qos property will enable the quality-of-service features of the basesink
  * which gather statistics about the real-time performance of the clock
  * synchronisation. For each buffer received in the sink, statistics are
- * gathered and a QOS event is send upstream with these numbers. This
+ * gathered and a QOS event is sent upstream with these numbers. This
  * information can then be used by upstream elements to reduce their processing
  * rate, for example.
  *
@@ -192,6 +192,12 @@ struct _GstBaseSinkPrivate
 
   /* latency stuff */
   GstClockTime latency;
+
+  /* if we already commited the state */
+  gboolean commited;
+
+  /* when we received EOS */
+  gboolean received_eos;
 };
 
 #define DO_RUNNING_AVG(avg,val,size) (((val) + ((size)-1) * (avg)) / (size))
@@ -205,13 +211,7 @@ struct _GstBaseSinkPrivate
 #define UPDATE_RUNNING_AVG_P(avg,val) DO_RUNNING_AVG(avg,val,16)
 #define UPDATE_RUNNING_AVG_N(avg,val) DO_RUNNING_AVG(avg,val,4)
 
-/* BaseSink signals and properties */
-enum
-{
-  /* FILL ME */
-  SIGNAL_HANDOFF,
-  LAST_SIGNAL
-};
+/* BaseSink properties */
 
 #define DEFAULT_SIZE 1024
 #define DEFAULT_CAN_ACTIVATE_PULL FALSE /* fixme: enable me */
@@ -888,9 +888,10 @@ gst_base_sink_commit_state (GstBaseSink * basesink)
 {
   /* commit state and proceed to next pending state */
   GstState current, next, pending, post_pending;
-  GstMessage *message;
   gboolean post_paused = FALSE;
+  gboolean post_async_done = FALSE;
   gboolean post_playing = FALSE;
+  gboolean sync;
 
   /* we are certainly not playing async anymore now */
   basesink->playing_async = FALSE;
@@ -900,6 +901,7 @@ gst_base_sink_commit_state (GstBaseSink * basesink)
   next = GST_STATE_NEXT (basesink);
   pending = GST_STATE_PENDING (basesink);
   post_pending = pending;
+  sync = basesink->sync;
 
   switch (pending) {
     case GST_STATE_PLAYING:
@@ -912,6 +914,8 @@ gst_base_sink_commit_state (GstBaseSink * basesink)
       GST_DEBUG_OBJECT (basesink, "commiting state to PLAYING");
 
       basesink->need_preroll = FALSE;
+      post_async_done = TRUE;
+      basesink->priv->commited = TRUE;
       post_playing = TRUE;
       /* post PAUSED too when we were READY */
       if (current == GST_STATE_READY) {
@@ -929,6 +933,8 @@ gst_base_sink_commit_state (GstBaseSink * basesink)
     case GST_STATE_PAUSED:
       GST_DEBUG_OBJECT (basesink, "commiting state to PAUSED");
       post_paused = TRUE;
+      post_async_done = TRUE;
+      basesink->priv->commited = TRUE;
       post_pending = GST_STATE_VOID_PENDING;
       break;
     case GST_STATE_READY:
@@ -947,19 +953,18 @@ gst_base_sink_commit_state (GstBaseSink * basesink)
   GST_OBJECT_UNLOCK (basesink);
 
   if (post_paused) {
-    message = gst_message_new_state_changed (GST_OBJECT_CAST (basesink),
-        current, next, post_pending);
-    gst_element_post_message (GST_ELEMENT_CAST (basesink), message);
+    gst_element_post_message (GST_ELEMENT_CAST (basesink),
+        gst_message_new_state_changed (GST_OBJECT_CAST (basesink),
+            current, next, post_pending));
+  }
+  if (post_async_done) {
+    gst_element_post_message (GST_ELEMENT_CAST (basesink),
+        gst_message_new_async_done (GST_OBJECT_CAST (basesink)));
   }
   if (post_playing) {
-    message = gst_message_new_state_changed (GST_OBJECT_CAST (basesink),
-        next, pending, GST_STATE_VOID_PENDING);
-    gst_element_post_message (GST_ELEMENT_CAST (basesink), message);
-  }
-  /* and mark dirty */
-  if (post_paused || post_playing) {
     gst_element_post_message (GST_ELEMENT_CAST (basesink),
-        gst_message_new_state_dirty (GST_OBJECT_CAST (basesink)));
+        gst_message_new_state_changed (GST_OBJECT_CAST (basesink),
+            next, pending, GST_STATE_VOID_PENDING));
   }
 
   GST_STATE_BROADCAST (basesink);
@@ -1329,8 +1334,8 @@ again:
     goto done;
 
   /* preroll done, we can sync since we are in PLAYING now. */
-  GST_DEBUG_OBJECT (basesink, "waiting for clock to reach %" GST_TIME_FORMAT,
-      GST_TIME_ARGS (rstart));
+  GST_DEBUG_OBJECT (basesink, "possibly waiting for clock to reach %"
+      GST_TIME_FORMAT, GST_TIME_ARGS (rstart));
 
   /* this function will return immediatly if start == -1, no clock
    * or sync is disabled with GST_CLOCK_BADTIME. */
@@ -1519,6 +1524,7 @@ gst_base_sink_reset_qos (GstBaseSink * sink)
   priv->avg_render = -1;
   priv->rendered = 0;
   priv->dropped = 0;
+
 }
 
 /* Checks if the object was scheduled too late.
@@ -1722,6 +1728,7 @@ gst_base_sink_render_object (GstBaseSink * basesink, GstPad * pad,
           /* configure the segment */
           gst_base_sink_configure_segment (basesink, pad, event,
               &basesink->segment);
+          break;
         default:
           break;
       }
@@ -1818,6 +1825,9 @@ gst_base_sink_queue_object_unlocked (GstBaseSink * basesink, GstPad * pad,
   gint length;
   GQueue *q;
 
+  if (G_UNLIKELY (basesink->priv->received_eos))
+    goto was_eos;
+
   if (G_UNLIKELY (basesink->need_preroll)) {
     if (G_LIKELY (prerollable))
       basesink->preroll_queued++;
@@ -1863,6 +1873,13 @@ gst_base_sink_queue_object_unlocked (GstBaseSink * basesink, GstPad * pad,
   return ret;
 
   /* special cases */
+was_eos:
+  {
+    GST_DEBUG_OBJECT (basesink,
+        "we are EOS, dropping object, return UNEXPECTED");
+    gst_mini_object_unref (obj);
+    return GST_FLOW_UNEXPECTED;
+  }
 preroll_failed:
   {
     GST_DEBUG_OBJECT (basesink, "preroll failed, reason %s",
@@ -1945,6 +1962,9 @@ gst_base_sink_event (GstPad * pad, GstEvent * event)
 
       if (G_UNLIKELY (ret != GST_FLOW_OK))
         result = FALSE;
+      else
+        /* we are now EOS, and refuse more buffers */
+        basesink->priv->received_eos = TRUE;
       break;
     }
     case GST_EVENT_NEWSEGMENT:
@@ -1953,19 +1973,25 @@ gst_base_sink_event (GstPad * pad, GstEvent * event)
 
       GST_DEBUG_OBJECT (basesink, "newsegment %p", event);
 
-      basesink->have_newsegment = TRUE;
-
-      /* the new segment is a non prerollable item and does not block anything,
-       * we need to configure the current clipping segment and insert the event 
-       * in the queue to serialize it with the buffers for rendering. */
-      gst_base_sink_configure_segment (basesink, pad, event,
-          basesink->abidata.ABI.clip_segment);
-
-      ret =
-          gst_base_sink_queue_object (basesink, pad,
-          GST_MINI_OBJECT_CAST (event), FALSE);
-      if (G_UNLIKELY (ret != GST_FLOW_OK))
+      if (G_UNLIKELY (basesink->priv->received_eos)) {
+        /* we can't accept anything when we are EOS */
         result = FALSE;
+        gst_event_unref (event);
+      } else {
+        /* the new segment is a non prerollable item and does not block anything,
+         * we need to configure the current clipping segment and insert the event 
+         * in the queue to serialize it with the buffers for rendering. */
+        gst_base_sink_configure_segment (basesink, pad, event,
+            basesink->abidata.ABI.clip_segment);
+
+        ret =
+            gst_base_sink_queue_object (basesink, pad,
+            GST_MINI_OBJECT_CAST (event), FALSE);
+        if (G_UNLIKELY (ret != GST_FLOW_OK))
+          result = FALSE;
+        else
+          basesink->have_newsegment = TRUE;
+      }
       break;
     }
     case GST_EVENT_FLUSH_START:
@@ -2005,6 +2031,8 @@ gst_base_sink_event (GstPad * pad, GstEvent * event)
       gst_segment_init (basesink->abidata.ABI.clip_segment,
           GST_FORMAT_UNDEFINED);
       basesink->have_newsegment = FALSE;
+      /* we flush out the EOS event as well now */
+      basesink->priv->received_eos = FALSE;
 
       gst_event_unref (event);
       break;
@@ -2049,12 +2077,13 @@ gst_base_sink_get_times (GstBaseSink * basesink, GstBuffer * buffer,
 
 /* must be called with PREROLL_LOCK */
 static gboolean
-gst_base_sink_is_prerolled (GstBaseSink * basesink)
+gst_base_sink_needs_preroll (GstBaseSink * basesink)
 {
-  gboolean res;
+  gboolean is_prerolled, res;
 
-  res = basesink->have_preroll || basesink->eos;
-  GST_DEBUG_OBJECT (basesink, "have_preroll: %d, EOS: %d => prerolled: %d",
+  is_prerolled = basesink->have_preroll || basesink->eos;
+  res = !is_prerolled && basesink->pad_mode != GST_ACTIVATE_PULL;
+  GST_DEBUG_OBJECT (basesink, "have_preroll: %d, EOS: %d => needs preroll: %d",
       basesink->have_preroll, basesink->eos, res);
   return res;
 }
@@ -2238,12 +2267,11 @@ static gboolean
 gst_base_sink_set_flushing (GstBaseSink * basesink, GstPad * pad,
     gboolean flushing)
 {
+  GstBaseSinkClass *bclass;
+
+  bclass = GST_BASE_SINK_GET_CLASS (basesink);
 
   if (flushing) {
-    GstBaseSinkClass *bclass;
-
-    bclass = GST_BASE_SINK_GET_CLASS (basesink);
-
     /* unlock any subclasses, we need to do this before grabbing the
      * PREROLL_LOCK since we hold this lock before going into ::render. */
     if (bclass->unlock)
@@ -2253,7 +2281,11 @@ gst_base_sink_set_flushing (GstBaseSink * basesink, GstPad * pad,
   GST_PAD_PREROLL_LOCK (pad);
   basesink->flushing = flushing;
   if (flushing) {
-    /* step 1, unblock clock sync (if any) or any other blocking thing */
+    /* step 1, now that we have the PREROLL lock, clear our unlock request */
+    if (bclass->unlock_stop)
+      bclass->unlock_stop (basesink);
+
+    /* step 2, unblock clock sync (if any) or any other blocking thing */
     basesink->need_preroll = TRUE;
     if (basesink->clock_id) {
       gst_clock_id_unschedule (basesink->clock_id);
@@ -2645,6 +2677,7 @@ gst_base_sink_get_position (GstBaseSink * basesink, GstFormat format,
           GST_TIME_FORMAT " + time %" GST_TIME_FORMAT,
           GST_TIME_ARGS (now), GST_TIME_ARGS (base),
           GST_TIME_ARGS (accum), GST_TIME_ARGS (time));
+      break;
     }
     default:
       /* cannot answer other than TIME, we return FALSE, which will
@@ -2724,6 +2757,13 @@ gst_base_sink_query (GstElement * element, GstQuery * query)
       gboolean live, us_live;
       GstClockTime min, max;
 
+      GST_PAD_PREROLL_LOCK (basesink->sinkpad);
+      if (gst_base_sink_needs_preroll (basesink)) {
+        GST_DEBUG_OBJECT (basesink, "not prerolled yet, can't report latency");
+        res = FALSE;
+        goto done;
+      }
+
       if ((res =
               gst_base_sink_query_latency (basesink, &live, &us_live, &min,
                   &max))) {
@@ -2737,6 +2777,8 @@ gst_base_sink_query (GstElement * element, GstQuery * query)
         }
         gst_query_set_latency (query, live, min, max);
       }
+    done:
+      GST_PAD_PREROLL_UNLOCK (basesink->sinkpad);
       break;
     }
     case GST_QUERY_JITTER:
@@ -2780,6 +2822,7 @@ gst_base_sink_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       /* need to complete preroll before this state change completes, there
        * is no data flow in READY so we can safely assume we need to preroll. */
+      GST_PAD_PREROLL_LOCK (basesink->sinkpad);
       GST_DEBUG_OBJECT (basesink, "READY to PAUSED, need preroll");
       gst_segment_init (&basesink->segment, GST_FORMAT_UNDEFINED);
       gst_segment_init (basesink->abidata.ABI.clip_segment,
@@ -2794,12 +2837,17 @@ gst_base_sink_change_state (GstElement * element, GstStateChange transition)
       basesink->priv->eos_rtime = -1;
       basesink->priv->latency = 0;
       basesink->eos = FALSE;
+      basesink->priv->received_eos = FALSE;
       gst_base_sink_reset_qos (basesink);
+      basesink->priv->commited = FALSE;
       ret = GST_STATE_CHANGE_ASYNC;
+      gst_element_post_message (GST_ELEMENT_CAST (basesink),
+          gst_message_new_async_start (GST_OBJECT_CAST (basesink), FALSE));
+      GST_PAD_PREROLL_UNLOCK (basesink->sinkpad);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       GST_PAD_PREROLL_LOCK (basesink->sinkpad);
-      if (gst_base_sink_is_prerolled (basesink)) {
+      if (!gst_base_sink_needs_preroll (basesink)) {
         /* no preroll needed anymore now. */
         GST_DEBUG_OBJECT (basesink, "PAUSED to PLAYING, don't need preroll");
         basesink->playing_async = FALSE;
@@ -2817,7 +2865,10 @@ gst_base_sink_change_state (GstElement * element, GstStateChange transition)
         GST_DEBUG_OBJECT (basesink, "PAUSED to PLAYING, need preroll");
         basesink->need_preroll = TRUE;
         basesink->playing_async = TRUE;
+        basesink->priv->commited = FALSE;
         ret = GST_STATE_CHANGE_ASYNC;
+        gst_element_post_message (GST_ELEMENT_CAST (basesink),
+            gst_message_new_async_start (GST_OBJECT_CAST (basesink), FALSE));
       }
       GST_PAD_PREROLL_UNLOCK (basesink->sinkpad);
       break;
@@ -2834,6 +2885,19 @@ gst_base_sink_change_state (GstElement * element, GstStateChange transition)
   }
 
   switch (transition) {
+    case GST_STATE_CHANGE_READY_TO_PAUSED:
+      /* note that this is the upward case, which doesn't follow most
+         patterns */
+      if (basesink->pad_mode == GST_ACTIVATE_PULL) {
+        GST_DEBUG_OBJECT (basesink, "basesink activated in pull mode, "
+            "returning SUCCESS directly");
+        GST_PAD_PREROLL_LOCK (basesink->sinkpad);
+        gst_element_post_message (GST_ELEMENT_CAST (basesink),
+            gst_message_new_async_done (GST_OBJECT_CAST (basesink)));
+        GST_PAD_PREROLL_UNLOCK (basesink->sinkpad);
+        ret = GST_STATE_CHANGE_SUCCESS;
+      }
+      break;
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       GST_DEBUG_OBJECT (basesink, "PLAYING to PAUSED");
       /* FIXME, make sure we cannot enter _render first */
@@ -2844,6 +2908,10 @@ gst_base_sink_change_state (GstElement * element, GstStateChange transition)
         bclass->unlock (basesink);
 
       GST_PAD_PREROLL_LOCK (basesink->sinkpad);
+      /* now that we have the PREROLL lock, clear our unlock request */
+      if (bclass->unlock_stop)
+        bclass->unlock_stop (basesink);
+
       basesink->need_preroll = TRUE;
       if (basesink->clock_id) {
         gst_clock_id_unschedule (basesink->clock_id);
@@ -2851,13 +2919,16 @@ gst_base_sink_change_state (GstElement * element, GstStateChange transition)
 
       /* if we don't have a preroll buffer we need to wait for a preroll and
        * return ASYNC. */
-      if (gst_base_sink_is_prerolled (basesink)) {
+      if (!gst_base_sink_needs_preroll (basesink)) {
         GST_DEBUG_OBJECT (basesink, "PLAYING to PAUSED, we are prerolled");
         basesink->playing_async = FALSE;
       } else {
         GST_DEBUG_OBJECT (basesink, "PLAYING to PAUSED, need preroll");
         basesink->playing_async = TRUE;
+        basesink->priv->commited = FALSE;
         ret = GST_STATE_CHANGE_ASYNC;
+        gst_element_post_message (GST_ELEMENT_CAST (basesink),
+            gst_message_new_async_start (GST_OBJECT_CAST (basesink), FALSE));
       }
       GST_DEBUG_OBJECT (basesink, "rendered: %" G_GUINT64_FORMAT
           ", dropped: %" G_GUINT64_FORMAT, basesink->priv->rendered,
@@ -2867,8 +2938,24 @@ gst_base_sink_change_state (GstElement * element, GstStateChange transition)
       GST_PAD_PREROLL_UNLOCK (basesink->sinkpad);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+      GST_PAD_PREROLL_LOCK (basesink->sinkpad);
+      if (!basesink->priv->commited) {
+        GST_DEBUG_OBJECT (basesink, "PAUSED to READY, posting async-done");
+
+        gst_element_post_message (GST_ELEMENT_CAST (basesink),
+            gst_message_new_state_changed (GST_OBJECT_CAST (basesink),
+                GST_STATE_PLAYING, GST_STATE_PAUSED, GST_STATE_READY));
+
+        gst_element_post_message (GST_ELEMENT_CAST (basesink),
+            gst_message_new_async_done (GST_OBJECT_CAST (basesink)));
+
+        basesink->priv->commited = TRUE;
+      } else {
+        GST_DEBUG_OBJECT (basesink, "PAUSED to READY, don't need_preroll");
+      }
       basesink->priv->current_sstart = 0;
       basesink->priv->current_sstop = 0;
+      GST_PAD_PREROLL_UNLOCK (basesink->sinkpad);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       if (bclass->stop)
