@@ -288,8 +288,11 @@ static gboolean gst_base_src_default_negotiate (GstBaseSrc * basesrc);
 static gboolean gst_base_src_default_do_seek (GstBaseSrc * src,
     GstSegment * segment);
 static gboolean gst_base_src_default_query (GstBaseSrc * src, GstQuery * query);
+static gboolean gst_base_src_default_prepare_seek_segment (GstBaseSrc * src,
+    GstEvent * event, GstSegment * segment);
 
 static gboolean gst_base_src_unlock (GstBaseSrc * basesrc);
+static gboolean gst_base_src_unlock_stop (GstBaseSrc * basesrc);
 static gboolean gst_base_src_start (GstBaseSrc * basesrc);
 static gboolean gst_base_src_stop (GstBaseSrc * basesrc);
 
@@ -352,6 +355,8 @@ gst_base_src_class_init (GstBaseSrcClass * klass)
   klass->query = GST_DEBUG_FUNCPTR (gst_base_src_default_query);
   klass->check_get_range =
       GST_DEBUG_FUNCPTR (gst_base_src_default_check_get_range);
+  klass->prepare_seek_segment =
+      GST_DEBUG_FUNCPTR (gst_base_src_default_prepare_seek_segment);
 }
 
 static void
@@ -541,6 +546,40 @@ gst_base_src_set_format (GstBaseSrc * src, GstFormat format)
   gst_segment_init (&src->segment, format);
 }
 
+/**
+ * gst_base_src_query_latency:
+ * @src: the source
+ * @live: if the source is live
+ * @min_latency: the min latency of the source
+ * @max_latency: the max latency of the source
+ *
+ * Query the source for the latency parameters. @live will be TRUE when @src is
+ * configured as a live source. @min_latency will be set as the latency between
+ * calling the create function and the timestamp on the resulting buffer.
+ * @max_latency is always the undefined value of -1.
+ *
+ * This function is mostly used by subclasses. 
+ *
+ * Returns: TRUE if the query succeeded.
+ *
+ * Since: 0.10.13
+ */
+gboolean
+gst_base_src_query_latency (GstBaseSrc * src, gboolean * live,
+    GstClockTime * min_latency, GstClockTime * max_latency)
+{
+  GST_LIVE_LOCK (src);
+  if (live)
+    *live = src->is_live;
+  if (min_latency)
+    *min_latency = 0;
+  if (max_latency)
+    *max_latency = -1;
+  GST_LIVE_UNLOCK (src);
+
+  return TRUE;
+}
+
 static gboolean
 gst_base_src_setcaps (GstPad * pad, GstCaps * caps)
 {
@@ -655,6 +694,9 @@ gst_base_src_default_query (GstBaseSrc * src, GstQuery * query)
       GstFormat format;
 
       gst_query_parse_duration (query, &format, NULL);
+
+      GST_DEBUG_OBJECT (src, "duration query in format %s",
+          gst_format_get_name (format));
       switch (format) {
         case GST_FORMAT_PERCENT:
           gst_query_set_duration (query, GST_FORMAT_PERCENT,
@@ -735,20 +777,24 @@ gst_base_src_default_query (GstBaseSrc * src, GstQuery * query)
       break;
     }
     case GST_QUERY_LATENCY:
-      /* we can only report the fact that we are live or not, we know nothing
-       * about latency. Subclasses should override and implement something
-       * usefull */
-      GST_LIVE_LOCK (src);
-      gst_query_set_latency (query, src->is_live, 0, -1);
-      GST_LIVE_UNLOCK (src);
-      res = TRUE;
+    {
+      GstClockTime min, max;
+      gboolean live;
+
+      /* Subclasses should override and implement something usefull */
+      res = gst_base_src_query_latency (src, &live, &min, &max);
+
+      gst_query_set_latency (query, live, min, max);
       break;
+    }
     case GST_QUERY_JITTER:
     case GST_QUERY_RATE:
     default:
       res = FALSE;
       break;
   }
+  GST_DEBUG_OBJECT (src, "query %s returns %d", GST_QUERY_TYPE_NAME (query),
+      res);
   return res;
 }
 
@@ -807,6 +853,84 @@ gst_base_src_do_seek (GstBaseSrc * src, GstSegment * segment)
   return result;
 }
 
+#define SEEK_TYPE_IS_RELATIVE(t) (((t) != GST_SEEK_TYPE_NONE) && ((t) != GST_SEEK_TYPE_SET))
+
+static gboolean
+gst_base_src_default_prepare_seek_segment (GstBaseSrc * src, GstEvent * event,
+    GstSegment * segment)
+{
+  /* By default, we try one of 2 things:
+   *   - For absolute seek positions, convert the requested position to our 
+   *     configured processing format and place it in the output segment \
+   *   - For relative seek positions, convert our current (input) values to the
+   *     seek format, adjust by the relative seek offset and then convert back to
+   *     the processing format
+   */
+  GstSeekType cur_type, stop_type;
+  gint64 cur, stop;
+  GstSeekFlags flags;
+  GstFormat seek_format, dest_format;
+  gdouble rate;
+  gboolean update;
+  gboolean res = TRUE;
+
+  gst_event_parse_seek (event, &rate, &seek_format, &flags,
+      &cur_type, &cur, &stop_type, &stop);
+  dest_format = segment->format;
+
+  if (seek_format == dest_format) {
+    gst_segment_set_seek (segment, rate, seek_format, flags,
+        cur_type, cur, stop_type, stop, &update);
+    return TRUE;
+  }
+
+  if (cur_type != GST_SEEK_TYPE_NONE) {
+    /* FIXME: Handle seek_cur & seek_end by converting the input segment vals */
+    res =
+        gst_pad_query_convert (src->srcpad, seek_format, cur, &dest_format,
+        &cur);
+    cur_type = GST_SEEK_TYPE_SET;
+  }
+
+  if (res && stop_type != GST_SEEK_TYPE_NONE) {
+    /* FIXME: Handle seek_cur & seek_end by converting the input segment vals */
+    res =
+        gst_pad_query_convert (src->srcpad, seek_format, stop, &dest_format,
+        &stop);
+    stop_type = GST_SEEK_TYPE_SET;
+  }
+
+  /* And finally, configure our output segment in the desired format */
+  gst_segment_set_seek (segment, rate, dest_format, flags, cur_type, cur,
+      stop_type, stop, &update);
+
+  if (!res)
+    goto no_format;
+
+  return res;
+
+no_format:
+  {
+    GST_DEBUG_OBJECT (src, "undefined format given, seek aborted.");
+    return FALSE;
+  }
+}
+
+static gboolean
+gst_base_src_prepare_seek_segment (GstBaseSrc * src, GstEvent * event,
+    GstSegment * seeksegment)
+{
+  GstBaseSrcClass *bclass;
+  gboolean result = FALSE;
+
+  bclass = GST_BASE_SRC_GET_CLASS (src);
+
+  if (bclass->prepare_seek_segment)
+    result = bclass->prepare_seek_segment (src, event, seeksegment);
+
+  return result;
+}
+
 /* this code implements the seeking. It is a good example
  * handling all cases.
  *
@@ -850,50 +974,55 @@ gst_base_src_do_seek (GstBaseSrc * src, GstSegment * segment)
  * when we reach the segment.stop we have to post a segment.done
  * instead of EOS when doing a segment seek.
  */
-/* FIXME, we have the unlock gboolean here because most current implementations
- * (fdsrc, -base/gst/tcp/, ...) not only unlock when there is something to
- * unlock
+/* FIXME (0.11), we have the unlock gboolean here because most current 
+ * implementations (fdsrc, -base/gst/tcp/, ...) unconditionally unlock, even when
+ * the streaming thread isn't running, resulting in bogus unlocks later when it 
+ * starts. This is fixed by adding unlock_stop, but we should still avoid unlocking
+ * unnecessarily for backwards compatibility. Ergo, the unlock variable stays
+ * until 0.11
  */
 static gboolean
 gst_base_src_perform_seek (GstBaseSrc * src, GstEvent * event, gboolean unlock)
 {
-  gboolean res;
+  gboolean res = TRUE;
   gdouble rate;
-  GstFormat format;
+  GstFormat seek_format, dest_format;
   GstSeekFlags flags;
   GstSeekType cur_type, stop_type;
   gint64 cur, stop;
   gboolean flush;
   gboolean update;
+  gboolean relative_seek = FALSE;
+  gboolean seekseg_configured = FALSE;
   GstSegment seeksegment;
 
   GST_DEBUG_OBJECT (src, "doing seek");
 
+  dest_format = src->segment.format;
+
   if (event) {
-    gst_event_parse_seek (event, &rate, &format, &flags,
+    gst_event_parse_seek (event, &rate, &seek_format, &flags,
         &cur_type, &cur, &stop_type, &stop);
 
-    /* we have to have a format as the segment format. Try to convert
-     * if not. */
-    if (src->segment.format != format) {
-      GstFormat fmt;
+    relative_seek = SEEK_TYPE_IS_RELATIVE (cur_type) ||
+        SEEK_TYPE_IS_RELATIVE (stop_type);
 
-      fmt = src->segment.format;
-      res = TRUE;
-      if (cur_type != GST_SEEK_TYPE_NONE)
-        res = gst_pad_query_convert (src->srcpad, format, cur, &fmt, &cur);
-      if (res && stop_type != GST_SEEK_TYPE_NONE)
-        res = gst_pad_query_convert (src->srcpad, format, stop, &fmt, &stop);
-      if (!res)
-        goto no_format;
+    if (dest_format != seek_format && !relative_seek) {
+      /* If we have an ABSOLUTE position (SEEK_SET only), we can convert it
+       * here before taking the stream lock, otherwise we must convert it later,
+       * once we have the stream lock and can read the current position */
+      gst_segment_init (&seeksegment, dest_format);
 
-      format = fmt;
+      if (!gst_base_src_prepare_seek_segment (src, event, &seeksegment))
+        goto prepare_failed;
+
+      seekseg_configured = TRUE;
     }
-  } else {
-    flags = 0;
-  }
 
-  flush = flags & GST_SEEK_FLAG_FLUSH;
+    flush = flags & GST_SEEK_FLAG_FLUSH;
+  } else {
+    flush = FALSE;
+  }
 
   /* send flush start */
   if (flush)
@@ -910,22 +1039,45 @@ gst_base_src_perform_seek (GstBaseSrc * src, GstEvent * event, gboolean unlock)
    * because our peer is flushing. */
   GST_PAD_STREAM_LOCK (src->srcpad);
 
-  /* make copy into temp structure, we can only update the main one
-   * when the subclass actually could do the seek. */
-  memcpy (&seeksegment, &src->segment, sizeof (GstSegment));
+  if (unlock)
+    gst_base_src_unlock_stop (src);
 
-  /* now configure the seek segment */
-  if (event) {
-    gst_segment_set_seek (&seeksegment, rate, format, flags,
-        cur_type, cur, stop_type, stop, &update);
+  /* If we configured the seeksegment above, don't overwrite it now. Otherwise
+   * copy the current segment info into the temp segment that we can actually
+   * attempt the seek with. We only update the real segment if the seek suceeds. */
+  if (!seekseg_configured) {
+    memcpy (&seeksegment, &src->segment, sizeof (GstSegment));
+
+    /* now configure the final seek segment */
+    if (event) {
+      if (src->segment.format != seek_format) {
+        /* OK, here's where we give the subclass a chance to convert the relative
+         * seek into an absolute one in the processing format. We set up any
+         * absolute seek above, before taking the stream lock. */
+        if (!gst_base_src_prepare_seek_segment (src, event, &seeksegment)) {
+          GST_DEBUG_OBJECT (src, "Preparing the seek failed after flushing. "
+              "Aborting seek");
+          res = FALSE;
+        }
+      } else {
+        /* The seek format matches our processing format, no need to ask the
+         * the subclass to configure the segment. */
+        gst_segment_set_seek (&seeksegment, rate, seek_format, flags,
+            cur_type, cur, stop_type, stop, &update);
+      }
+    }
+    /* Else, no seek event passed, so we're just (re)starting the 
+       current segment. */
   }
 
-  GST_DEBUG_OBJECT (src, "segment configured from %" G_GINT64_FORMAT
-      " to %" G_GINT64_FORMAT ", position %" G_GINT64_FORMAT,
-      seeksegment.start, seeksegment.stop, seeksegment.last_stop);
+  if (res) {
+    GST_DEBUG_OBJECT (src, "segment configured from %" G_GINT64_FORMAT
+        " to %" G_GINT64_FORMAT ", position %" G_GINT64_FORMAT,
+        seeksegment.start, seeksegment.stop, seeksegment.last_stop);
 
-  /* do the seek, segment.last_stop contains new position. */
-  res = gst_base_src_do_seek (src, &seeksegment);
+    /* do the seek, segment.last_stop contains the new position. */
+    res = gst_base_src_do_seek (src, &seeksegment);
+  }
 
   /* and prepare to continue streaming */
   if (flush) {
@@ -947,6 +1099,14 @@ gst_base_src_perform_seek (GstBaseSrc * src, GstEvent * event, gboolean unlock)
         src->segment.start, src->segment.last_stop, src->segment.time);
   }
 
+  /* The subclass must have converted the segment to the processing format 
+   * by now */
+  if (res && seeksegment.format != dest_format) {
+    GST_DEBUG_OBJECT (src, "Subclass failed to prepare a seek segment "
+        "in the correct format. Aborting seek.");
+    res = FALSE;
+  }
+
   /* if successfull seek, we update our real segment and push
    * out the new segment. */
   if (res) {
@@ -989,11 +1149,10 @@ gst_base_src_perform_seek (GstBaseSrc * src, GstEvent * event, gboolean unlock)
   return res;
 
   /* ERROR */
-no_format:
-  {
-    GST_DEBUG_OBJECT (src, "undefined format given, seek aborted.");
-    return FALSE;
-  }
+prepare_failed:
+  GST_DEBUG_OBJECT (src, "Preparing the seek failed before flushing. "
+      "Aborting seek");
+  return FALSE;
 }
 
 static const GstQueryType *
@@ -1110,6 +1269,8 @@ gst_base_src_default_event (GstBaseSrc * src, GstEvent * event)
       result = gst_base_src_unlock (src);
       break;
     case GST_EVENT_FLUSH_STOP:
+      result = gst_base_src_unlock_stop (src);
+      break;
     default:
       result = TRUE;
       break;
@@ -1391,21 +1552,33 @@ gst_base_src_get_range (GstBaseSrc * src, guint64 offset, guint length,
   status = gst_base_src_do_sync (src, *buf);
   switch (status) {
     case GST_CLOCK_EARLY:
+      /* the buffer is too late. We currently don't drop the buffer. */
       GST_DEBUG_OBJECT (src, "buffer too late!, returning anyway");
       break;
     case GST_CLOCK_OK:
+      /* buffer synchronised properly */
       GST_DEBUG_OBJECT (src, "buffer ok");
       break;
-    default:
+    case GST_CLOCK_UNSCHEDULED:
       /* this case is triggered when we were waiting for the clock and
        * it got unlocked because we did a state change. We return 
        * WRONG_STATE in this case to stop the dataflow also get rid of the
        * produced buffer. */
-      GST_DEBUG_OBJECT (src, "clock returned %d, not returning", status);
+      GST_DEBUG_OBJECT (src,
+          "clock was unscheduled (%d), returning WRONG_STATE", status);
       gst_buffer_unref (*buf);
       *buf = NULL;
       ret = GST_FLOW_WRONG_STATE;
       break;
+    default:
+      /* all other result values are unexpected and errors */
+      GST_ELEMENT_ERROR (src, CORE, CLOCK,
+          (_("Internal clock error.")),
+          ("clock returned unexpected return value %d", status));
+      gst_buffer_unref (*buf);
+      *buf = NULL;
+      ret = GST_FLOW_ERROR;
+      break;
   }
 done:
   return ret;
@@ -1529,16 +1702,6 @@ gst_base_src_loop (GstPad * pad)
 
   src->priv->last_sent_eos = FALSE;
 
-  /* push events to close/start our segment before we do the real work. */
-  if (src->priv->close_segment) {
-    gst_pad_push_event (pad, src->priv->close_segment);
-    src->priv->close_segment = NULL;
-  }
-  if (src->priv->start_segment) {
-    gst_pad_push_event (pad, src->priv->start_segment);
-    src->priv->start_segment = NULL;
-  }
-
   /* if we operate in bytes, we can calculate an offset */
   if (src->segment.format == GST_FORMAT_BYTES)
     position = src->segment.last_stop;
@@ -1551,9 +1714,20 @@ gst_base_src_loop (GstPad * pad)
         gst_flow_get_name (ret));
     goto pause;
   }
+  /* this should not happen */
   if (G_UNLIKELY (buf == NULL))
     goto null_buffer;
 
+  /* push events to close/start our segment before we push the buffer. */
+  if (src->priv->close_segment) {
+    gst_pad_push_event (pad, src->priv->close_segment);
+    src->priv->close_segment = NULL;
+  }
+  if (src->priv->start_segment) {
+    gst_pad_push_event (pad, src->priv->start_segment);
+    src->priv->start_segment = NULL;
+  }
+
   /* figure out the new position */
   switch (src->segment.format) {
     case GST_FORMAT_BYTES:
@@ -1687,6 +1861,28 @@ gst_base_src_unlock (GstBaseSrc * basesrc)
   return result;
 }
 
+/* this will always be called between start() and stop(). So you can rely on
+ * resources allocated by start() and freed from stop(). This needs to be added
+ * to the docs at some point. */
+static gboolean
+gst_base_src_unlock_stop (GstBaseSrc * basesrc)
+{
+  GstBaseSrcClass *bclass;
+  gboolean result = TRUE;
+
+  GST_DEBUG_OBJECT (basesrc, "unlock stop");
+
+  /* Finish a previous unblock request, allowing subclasses to flush command
+   * queues or whatever they need to do */
+  bclass = GST_BASE_SRC_GET_CLASS (basesrc);
+  if (bclass->unlock_stop)
+    result = bclass->unlock_stop (basesrc);
+
+  GST_DEBUG_OBJECT (basesrc, "unlock stop done");
+
+  return result;
+}
+
 /* default negotiation code. 
  *
  * Take intersection between src and sink pads, take first
@@ -1810,6 +2006,7 @@ gst_base_src_start (GstBaseSrc * basesrc)
       result = FALSE;
       size = -1;
     }
+    GST_DEBUG_OBJECT (basesrc, "setting size %" G_GUINT64_FORMAT, size);
     /* only update the size when operating in bytes, subclass is supposed
      * to set duration in the start method for other formats */
     gst_segment_set_duration (&basesrc->segment, GST_FORMAT_BYTES, size);
@@ -1906,6 +2103,9 @@ gst_base_src_deactivate (GstBaseSrc * basesrc, GstPad * pad)
   /* step 2, make sure streaming finishes */
   result &= gst_pad_stop_task (pad);
 
+  /* step 3, clear the unblock condition */
+  result &= gst_base_src_unlock_stop (basesrc);
+
   return result;
 }
 
