@@ -79,11 +79,15 @@ GST_DEBUG_CATEGORY_STATIC (collect_pads_debug);
 
 GST_BOILERPLATE (GstCollectPads, gst_collect_pads, GstObject, GST_TYPE_OBJECT);
 
+static void gst_collect_pads_clear (GstCollectPads * pads,
+    GstCollectData * data);
 static GstFlowReturn gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer);
 static gboolean gst_collect_pads_event (GstPad * pad, GstEvent * event);
 static void gst_collect_pads_finalize (GObject * object);
 static void gst_collect_pads_init (GstCollectPads * pads,
     GstCollectPadsClass * g_class);
+static void ref_data (GstCollectData * data);
+static void unref_data (GstCollectData * data);
 
 static void
 gst_collect_pads_base_init (gpointer g_class)
@@ -133,14 +137,10 @@ gst_collect_pads_finalize (GObject * object)
   for (; collected; collected = g_slist_next (collected)) {
     GstCollectData *pdata = (GstCollectData *) collected->data;
 
-    if (pdata->pad) {
-      GST_DEBUG ("finalize pad %s:%s", GST_DEBUG_PAD_NAME (pdata->pad));
-      gst_object_unref (pdata->pad);
-      pdata->pad = NULL;
-    }
-    g_free (pdata);
+    unref_data (pdata);
   }
   /* Free pads list */
+  g_slist_foreach (pads->data, (GFunc) unref_data, NULL);
   g_slist_free (pads->data);
   g_slist_free (pads->abidata.ABI.pad_list);
 
@@ -190,11 +190,45 @@ gst_collect_pads_set_function (GstCollectPads * pads,
   GST_OBJECT_UNLOCK (pads);
 }
 
+static void
+ref_data (GstCollectData * data)
+{
+  g_assert (data != NULL);
+
+  g_atomic_int_inc (&(data->abidata.ABI.refcount));
+}
+
+static void
+unref_data (GstCollectData * data)
+{
+  GstCollectDataDestroyNotify destroy_notify;
+
+  g_assert (data != NULL);
+  g_assert (data->abidata.ABI.refcount > 0);
+
+  if (!g_atomic_int_dec_and_test (&(data->abidata.ABI.refcount)))
+    return;
+
+  /* FIXME: Ugly hack as we can't add more fields to GstCollectData */
+  destroy_notify = (GstCollectDataDestroyNotify)
+      g_object_get_data (G_OBJECT (data->pad),
+      "gst-collect-data-destroy-notify");
+
+  if (destroy_notify)
+    destroy_notify (data);
+
+  g_object_unref (data->pad);
+  if (data->buffer) {
+    gst_buffer_unref (data->buffer);
+  }
+  g_free (data);
+}
+
 /**
  * gst_collect_pads_add_pad:
  * @pads: the collectspads to use
  * @pad: the pad to add
- * @size: the size of the returned GstCollectData structure
+ * @size: the size of the returned #GstCollectData structure
  *
  * Add a pad to the collection of collect pads. The pad has to be
  * a sinkpad. The refcount of the pad is incremented. Use
@@ -204,6 +238,12 @@ gst_collect_pads_set_function (GstCollectPads * pads,
  * You specify a size for the returned #GstCollectData structure
  * so that you can use it to store additional information.
  *
+ * The pad will be automatically activated in push mode when @pads is
+ * started.
+ *
+ * This function calls gst_collect_pads_add_pad() passing a value of NULL
+ * for destroy_notify.
+ *
  * Returns: a new #GstCollectData to identify the new pad. Or NULL
  *   if wrong parameters are supplied.
  *
@@ -211,6 +251,44 @@ gst_collect_pads_set_function (GstCollectPads * pads,
  */
 GstCollectData *
 gst_collect_pads_add_pad (GstCollectPads * pads, GstPad * pad, guint size)
+{
+  return gst_collect_pads_add_pad_full (pads, pad, size, NULL);
+}
+
+/**
+ * gst_collect_pads_add_pad_full:
+ * @pads: the collectspads to use
+ * @pad: the pad to add
+ * @size: the size of the returned #GstCollectData structure
+ * @destroy_notify: function to be called before the returned #GstCollectData
+ * structure is freed
+ *
+ * Add a pad to the collection of collect pads. The pad has to be
+ * a sinkpad. The refcount of the pad is incremented. Use
+ * gst_collect_pads_remove_pad() to remove the pad from the collection
+ * again.
+ *
+ * You specify a size for the returned #GstCollectData structure
+ * so that you can use it to store additional information.
+ *
+ * You can also specify a #GstCollectDataDestroyNotify that will be called
+ * just before the #GstCollectData structure is freed. It is passed the
+ * pointer to the structure and should free any custom memory and resources
+ * allocated for it.
+ *
+ * The pad will be automatically activated in push mode when @pads is
+ * started.
+ *
+ * Since: 0.10.12
+ *
+ * Returns: a new #GstCollectData to identify the new pad. Or NULL
+ *   if wrong parameters are supplied.
+ *
+ * MT safe.
+ */
+GstCollectData *
+gst_collect_pads_add_pad_full (GstCollectPads * pads, GstPad * pad, guint size,
+    GstCollectDataDestroyNotify destroy_notify)
 {
   GstCollectData *data;
 
@@ -231,13 +309,23 @@ gst_collect_pads_add_pad (GstCollectPads * pads, GstPad * pad, guint size)
   data->abidata.ABI.flushing = FALSE;
   data->abidata.ABI.new_segment = FALSE;
   data->abidata.ABI.eos = FALSE;
+  data->abidata.ABI.refcount = 1;
+
+  /* FIXME: Ugly hack as we can't add more fields to GstCollectData */
+  g_object_set_data (G_OBJECT (pad), "gst-collect-data-destroy-notify",
+      destroy_notify);
 
   GST_COLLECT_PADS_PAD_LOCK (pads);
+  GST_OBJECT_LOCK (pad);
+  gst_pad_set_element_private (pad, data);
+  GST_OBJECT_UNLOCK (pad);
   pads->abidata.ABI.pad_list =
       g_slist_append (pads->abidata.ABI.pad_list, data);
   gst_pad_set_chain_function (pad, GST_DEBUG_FUNCPTR (gst_collect_pads_chain));
   gst_pad_set_event_function (pad, GST_DEBUG_FUNCPTR (gst_collect_pads_event));
-  gst_pad_set_element_private (pad, data);
+  /* activate the pad when needed */
+  if (pads->started)
+    gst_pad_set_active (pad, TRUE);
   pads->abidata.ABI.pad_cookie++;
   GST_COLLECT_PADS_PAD_UNLOCK (pads);
 
@@ -257,15 +345,20 @@ find_pad (GstCollectData * data, GstPad * pad)
  * @pads: the collectspads to use
  * @pad: the pad to remove
  *
- * Remove a pad from the collection of collect pads.
+ * Remove a pad from the collection of collect pads. This function will also
+ * free the #GstCollectData and all the resources that were allocated with 
+ * gst_collect_pads_add_pad().
  *
- * Returns: TRUE if the pad could be removed.
+ * The pad will be deactivated automatically when @pads is stopped.
+ *
+ * Returns: %TRUE if the pad could be removed.
  *
  * MT safe.
  */
 gboolean
 gst_collect_pads_remove_pad (GstCollectPads * pads, GstPad * pad)
 {
+  GstCollectData *data;
   GSList *list;
 
   g_return_val_if_fail (pads != NULL, FALSE);
@@ -282,30 +375,46 @@ gst_collect_pads_remove_pad (GstCollectPads * pads, GstPad * pad)
   if (!list)
     goto unknown_pad;
 
-  GST_DEBUG ("found pad %s:%s at %p", GST_DEBUG_PAD_NAME (pad), list->data);
+  data = (GstCollectData *) list->data;
+
+  GST_DEBUG ("found pad %s:%s at %p", GST_DEBUG_PAD_NAME (pad), data);
+
   /* clear the stuff we configured */
   gst_pad_set_chain_function (pad, NULL);
   gst_pad_set_event_function (pad, NULL);
-  /* FIXME, check that freeing the private data does not causes
-   * crashes in the streaming thread */
+  GST_OBJECT_LOCK (pad);
   gst_pad_set_element_private (pad, NULL);
+  GST_OBJECT_UNLOCK (pad);
 
-  /* backward compat, also remove from data if stopped */
+  /* backward compat, also remove from data if stopped, note that this function
+   * can only be called when we are stopped because we don't take the LOCK to
+   * protect the pads->data list. */
   if (!pads->started) {
     GSList *dlist;
 
     dlist = g_slist_find_custom (pads->data, pad, (GCompareFunc) find_pad);
     if (dlist) {
+      GstCollectData *pdata = dlist->data;
+
       pads->data = g_slist_delete_link (pads->data, dlist);
+      unref_data (pdata);
     }
   }
-  gst_object_unref (pad);
-  g_free (list->data);
+  /* remove from the pad list */
   pads->abidata.ABI.pad_list =
       g_slist_delete_link (pads->abidata.ABI.pad_list, list);
   pads->abidata.ABI.pad_cookie++;
+
   /* signal waiters because something changed */
   GST_COLLECT_PADS_BROADCAST (pads);
+
+  /* deactivate the pad when needed */
+  if (!pads->started)
+    gst_pad_set_active (pad, FALSE);
+
+  /* clean and free the collect data */
+  unref_data (data);
+
   GST_COLLECT_PADS_PAD_UNLOCK (pads);
 
   return TRUE;
@@ -327,7 +436,7 @@ unknown_pad:
  *
  * This function is currently not implemented.
  *
- * Returns: TRUE if the pad is active.
+ * Returns: %TRUE if the pad is active.
  *
  * MT safe.
  */
@@ -349,11 +458,11 @@ gst_collect_pads_is_active (GstCollectPads * pads, GstPad * pad)
  * @pads: the collectspads to use
  *
  * Collect data on all pads. This function is usually called
- * from a GstTask function in an element. 
+ * from a #GstTask function in an element. 
  *
  * This function is currently not implemented.
  *
- * Returns: GstFlowReturn of the operation.
+ * Returns: #GstFlowReturn of the operation.
  *
  * MT safe.
  */
@@ -379,7 +488,7 @@ gst_collect_pads_collect (GstCollectPads * pads)
  *
  * This function is currently not implemented.
  *
- * Returns: GstFlowReturn of the operation.
+ * Returns: #GstFlowReturn of the operation.
  *
  * MT safe.
  */
@@ -417,6 +526,7 @@ gst_collect_pads_set_flushing_unlocked (GstCollectPads * pads,
       else
         GST_PAD_UNSET_FLUSHING (cdata->pad);
       cdata->abidata.ABI.flushing = flushing;
+      gst_collect_pads_clear (pads, cdata);
       GST_OBJECT_UNLOCK (cdata->pad);
     }
   }
@@ -429,7 +539,7 @@ gst_collect_pads_set_flushing_unlocked (GstCollectPads * pads,
  *
  * Change the flushing state of all the pads in the collection. No pad
  * is able to accept anymore data when @flushing is %TRUE. Calling this
- * function with @flushing %TRUE makes @pads accept data again.
+ * function with @flushing %FALSE makes @pads accept data again.
  *
  * MT safe.
  *
@@ -534,7 +644,7 @@ gst_collect_pads_stop (GstCollectPads * pads)
   }
 
   GST_COLLECT_PADS_PAD_UNLOCK (pads);
-  /* Wake them up so then can end the chain functions. */
+  /* Wake them up so they can end the chain functions. */
   GST_COLLECT_PADS_BROADCAST (pads);
 
   GST_OBJECT_UNLOCK (pads);
@@ -590,15 +700,13 @@ GstBuffer *
 gst_collect_pads_pop (GstCollectPads * pads, GstCollectData * data)
 {
   GstBuffer *result;
-  GstBuffer **buffer_p;
 
   g_return_val_if_fail (pads != NULL, NULL);
   g_return_val_if_fail (GST_IS_COLLECT_PADS (pads), NULL);
   g_return_val_if_fail (data != NULL, NULL);
 
   if ((result = data->buffer)) {
-    buffer_p = &data->buffer;
-    gst_buffer_replace (buffer_p, NULL);
+    data->buffer = NULL;
     data->pos = 0;
     /* one less pad with queued data now */
     pads->queuedpads--;
@@ -612,6 +720,17 @@ gst_collect_pads_pop (GstCollectPads * pads, GstCollectData * data)
   return result;
 }
 
+/* pop and unref the currently queued buffer, should e called with the LOCK
+ * helt. */
+static void
+gst_collect_pads_clear (GstCollectPads * pads, GstCollectData * data)
+{
+  GstBuffer *buf;
+
+  if ((buf = gst_collect_pads_pop (pads, data)))
+    gst_buffer_unref (buf);
+}
+
 /**
  * gst_collect_pads_available:
  * @pads: the collectspads to query
@@ -623,7 +742,7 @@ gst_collect_pads_pop (GstCollectPads * pads, GstCollectData * data)
  * This function should be called with @pads LOCK held, such as
  * in the callback.
  *
- * Returns: The maximum number of bytes queued on all pad. This function
+ * Returns: The maximum number of bytes queued on all pads. This function
  * returns 0 if a pad has no queued buffer.
  *
  * MT safe.
@@ -755,13 +874,9 @@ gst_collect_pads_flush (GstCollectPads * pads, GstCollectData * data,
 
   data->pos += size;
 
-  if (data->pos >= GST_BUFFER_SIZE (buffer)) {
-    GstBuffer *buf;
-
-    /* _pop will also reset data->pos to 0 */
-    buf = gst_collect_pads_pop (pads, data);
-    gst_buffer_unref (buf);
-  }
+  if (data->pos >= GST_BUFFER_SIZE (buffer))
+    /* _clear will also reset data->pos to 0 */
+    gst_collect_pads_clear (pads, data);
 
   return flushsize;
 }
@@ -784,6 +899,7 @@ gst_collect_pads_check_pads (GstCollectPads * pads)
     GSList *collected;
 
     /* clear list and stats */
+    g_slist_foreach (pads->data, (GFunc) unref_data, NULL);
     g_slist_free (pads->data);
     pads->data = NULL;
     pads->numpads = 0;
@@ -804,6 +920,7 @@ gst_collect_pads_check_pads (GstCollectPads * pads)
         pads->eospads++;
 
       /* add to the list of pads to collect */
+      ref_data (data);
       pads->data = g_slist_prepend (pads->data, data);
     }
     /* and update the cookie */
@@ -816,7 +933,7 @@ gst_collect_pads_check_pads (GstCollectPads * pads)
  *
  * Should be called with LOCK.
  *
- * Returns: The GstFlowReturn of collection.
+ * Returns: The #GstFlowReturn of collection.
  */
 static GstFlowReturn
 gst_collect_pads_check_collected (GstCollectPads * pads)
@@ -872,9 +989,12 @@ gst_collect_pads_event (GstPad * pad, GstEvent * event)
   GstCollectPads *pads;
 
   /* some magic to get the managing collect_pads */
+  GST_OBJECT_LOCK (pad);
   data = (GstCollectData *) gst_pad_get_element_private (pad);
   if (G_UNLIKELY (data == NULL))
-    goto not_ours;
+    goto pad_removed;
+  ref_data (data);
+  GST_OBJECT_UNLOCK (pad);
 
   res = TRUE;
 
@@ -894,7 +1014,7 @@ gst_collect_pads_event (GstPad * pad, GstEvent * event)
        * non-flushing block again */
       GST_OBJECT_LOCK (pads);
       data->abidata.ABI.flushing = TRUE;
-      GST_COLLECT_PADS_BROADCAST (pads);
+      gst_collect_pads_clear (pads, data);
       GST_OBJECT_UNLOCK (pads);
 
       /* event already cleaned up by forwarding */
@@ -905,7 +1025,7 @@ gst_collect_pads_event (GstPad * pad, GstEvent * event)
       /* flush the 1 buffer queue */
       GST_OBJECT_LOCK (pads);
       data->abidata.ABI.flushing = FALSE;
-      gst_collect_pads_pop (pads, data);
+      gst_collect_pads_clear (pads, data);
       /* we need new segment info after the flush */
       gst_segment_init (&data->segment, GST_FORMAT_UNDEFINED);
       data->abidata.ABI.new_segment = FALSE;
@@ -953,9 +1073,6 @@ gst_collect_pads_event (GstPad * pad, GstEvent * event)
           ", stop %" GST_TIME_FORMAT, GST_TIME_ARGS (start),
           GST_TIME_ARGS (stop));
 
-      if (data->segment.format != format)
-        gst_segment_init (&data->segment, format);
-
       gst_segment_set_newsegment_full (&data->segment, update, rate, arate,
           format, start, stop, time);
 
@@ -965,14 +1082,14 @@ gst_collect_pads_event (GstPad * pad, GstEvent * event)
        * accumulated and this is certainly not what we want. */
       gst_event_unref (event);
       /* FIXME: collect-pads based elements need to create their own newsegment
-         event (and only one really)
-         (a) make the segment part of the GstCollectData structure of each pad,
-         so you can just check that once you have a buffer queued on that pad,
-         (b) you can override a pad's event function with your own,
-         catch the newsegment event and then pass it on to the original
-         gstcollectpads event function
-         (that's what avimux does for something IIRC)
-         see #340060
+       * event (and only one really)
+       * (a) make the segment part of the GstCollectData structure of each pad,
+       * so you can just check that once you have a buffer queued on that pad,
+       * (b) you can override a pad's event function with your own,
+       * catch the newsegment event and then pass it on to the original
+       * gstcollectpads event function
+       * (that's what avimux does for something IIRC)
+       * see #340060
        */
       goto done;
     }
@@ -985,12 +1102,14 @@ forward:
   res = gst_pad_event_default (pad, event);
 
 done:
+  unref_data (data);
   return res;
 
   /* ERRORS */
-not_ours:
+pad_removed:
   {
-    GST_WARNING ("not our pad");
+    GST_DEBUG ("%s got removed from collectpads", GST_OBJECT_NAME (pad));
+    GST_OBJECT_UNLOCK (pad);
     return FALSE;
   }
 }
@@ -1013,15 +1132,17 @@ gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer)
   GST_DEBUG ("Got buffer for pad %s:%s", GST_DEBUG_PAD_NAME (pad));
 
   /* some magic to get the managing collect_pads */
+  GST_OBJECT_LOCK (pad);
   data = (GstCollectData *) gst_pad_get_element_private (pad);
   if (G_UNLIKELY (data == NULL))
-    goto not_ours;
+    goto no_data;
+  ref_data (data);
+  GST_OBJECT_UNLOCK (pad);
 
   pads = data->collect;
   size = GST_BUFFER_SIZE (buffer);
 
   GST_OBJECT_LOCK (pads);
-
   /* if not started, bail out */
   if (G_UNLIKELY (!pads->started))
     goto not_started;
@@ -1062,13 +1183,22 @@ gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer)
     if (data->buffer == NULL)
       break;
 
+    /* Check if we got removed in the mean time, FIXME, this is racy.
+     * Between this check and the _WAIT, the pad could be removed which will
+     * makes us hang in the _WAIT. */
+    GST_OBJECT_LOCK (pad);
+    if (G_UNLIKELY (gst_pad_get_element_private (pad) == NULL))
+      goto pad_removed;
+    GST_OBJECT_UNLOCK (pad);
+
     GST_DEBUG ("Pad %s:%s has a buffer queued, waiting",
         GST_DEBUG_PAD_NAME (pad));
 
     /* wait to be collected, this must happen from another thread triggered
      * by the _chain function of another pad. We release the lock so we
      * can get stopped or flushed as well. We can however not get EOS
-     * because we still hold the STREAM_LOCK. */
+     * because we still hold the STREAM_LOCK. 
+     */
     GST_COLLECT_PADS_WAIT (pads);
 
     GST_DEBUG ("Pad %s:%s resuming", GST_DEBUG_PAD_NAME (pad));
@@ -1082,43 +1212,55 @@ gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer)
   }
   while (data->buffer != NULL);
 
+unlock_done:
   GST_OBJECT_UNLOCK (pads);
-
+  unref_data (data);
+  gst_buffer_unref (buffer);
   return ret;
 
-  /* ERRORS */
-not_ours:
+pad_removed:
   {
-    /* pretty fatal this one, can't post an error though... */
-    GST_WARNING ("not our pad");
-    return GST_FLOW_ERROR;
+    GST_WARNING ("%s got removed from collectpads", GST_OBJECT_NAME (pad));
+    GST_OBJECT_UNLOCK (pad);
+    ret = GST_FLOW_NOT_LINKED;
+    goto unlock_done;
+  }
+  /* ERRORS */
+no_data:
+  {
+    GST_DEBUG ("%s got removed from collectpads", GST_OBJECT_NAME (pad));
+    GST_OBJECT_UNLOCK (pad);
+    gst_buffer_unref (buffer);
+    return GST_FLOW_NOT_LINKED;
   }
 not_started:
   {
-    GST_OBJECT_UNLOCK (pads);
     GST_DEBUG ("not started");
-    return GST_FLOW_WRONG_STATE;
+    gst_collect_pads_clear (pads, data);
+    ret = GST_FLOW_WRONG_STATE;
+    goto unlock_done;
   }
 flushing:
   {
-    GST_OBJECT_UNLOCK (pads);
     GST_DEBUG ("pad %s:%s is flushing", GST_DEBUG_PAD_NAME (pad));
-    return GST_FLOW_WRONG_STATE;
+    gst_collect_pads_clear (pads, data);
+    ret = GST_FLOW_WRONG_STATE;
+    goto unlock_done;
   }
 unexpected:
   {
-    GST_OBJECT_UNLOCK (pads);
     /* we should not post an error for this, just inform upstream that
      * we don't expect anything anymore */
     GST_DEBUG ("pad %s:%s is eos", GST_DEBUG_PAD_NAME (pad));
-    return GST_FLOW_UNEXPECTED;
+    ret = GST_FLOW_UNEXPECTED;
+    goto unlock_done;
   }
 error:
   {
-    GST_OBJECT_UNLOCK (pads);
     /* we print the error, the element should post a reasonable error
      * message for fatal errors */
     GST_DEBUG ("collect failed, reason %d (%s)", ret, gst_flow_get_name (ret));
-    return ret;
+    gst_collect_pads_clear (pads, data);
+    goto unlock_done;
   }
 }
