@@ -26,11 +26,14 @@
 #include <signal.h>
 #include <string.h>
 #include <stdlib.h>
+#include <math.h>
 
 #include <check.h>
 
 #include <gst/gst.h>
 
+G_BEGIN_DECLS
+
 GST_DEBUG_CATEGORY_EXTERN (check_debug);
 #define GST_CAT_DEFAULT check_debug
 
@@ -103,6 +106,15 @@ static void __testname ()\
 
 
 /* additional fail macros */
+/**
+ * fail_unless_equals_int:
+ * @a: a #gint value or expression
+ * @b: a #gint value or expression
+ *
+ * This macro checks that @a and @b are equal and aborts if this is not the
+ * case, printing both expressions and the values they evaluated to. This
+ * macro is for use in unit tests.
+ */
 #define fail_unless_equals_int(a, b)					\
 G_STMT_START {								\
   int first = a;							\
@@ -110,8 +122,26 @@ G_STMT_START {								\
   fail_unless(first == second,						\
     "'" #a "' (%d) is not equal to '" #b"' (%d)", first, second);	\
 } G_STMT_END;
+/**
+ * assert_equals_int:
+ * @a: a #gint value or expression
+ * @b: a #gint value or expression
+ *
+ * This macro checks that @a and @b are equal and aborts if this is not the
+ * case, printing both expressions and the values they evaluated to. This
+ * macro is for use in unit tests.
+ */
 #define assert_equals_int(a, b) fail_unless_equals_int(a, b)
 
+/**
+ * fail_unless_equals_uint64:
+ * @a: a #guint64 value or expression
+ * @b: a #guint64 value or expression
+ *
+ * This macro checks that @a and @b are equal and aborts if this is not the
+ * case, printing both expressions and the values they evaluated to. This
+ * macro is for use in unit tests.
+ */
 #define fail_unless_equals_uint64(a, b)					\
 G_STMT_START {								\
   guint64 first = a;							\
@@ -120,8 +150,26 @@ G_STMT_START {								\
     "'" #a "' (%" G_GUINT64_FORMAT ") is not equal to '" #b"' (%"	\
     G_GUINT64_FORMAT ")", first, second);				\
 } G_STMT_END;
+/**
+ * assert_equals_uint64:
+ * @a: a #guint64 value or expression
+ * @b: a #guint64 value or expression
+ *
+ * This macro checks that @a and @b are equal and aborts if this is not the
+ * case, printing both expressions and the values they evaluated to. This
+ * macro is for use in unit tests.
+ */
 #define assert_equals_uint64(a, b) fail_unless_equals_uint64(a, b)
 
+/**
+ * fail_unless_equals_string:
+ * @a: a string literal or expression
+ * @b: a string literal or expression
+ *
+ * This macro checks that @a and @b are equal (as per strcmp) and aborts if
+ * this is not the case, printing both expressions and the values they
+ * evaluated to. This macro is for use in unit tests.
+ */
 #define fail_unless_equals_string(a, b)                             \
 G_STMT_START {                                                      \
   const gchar * first = a;                                          \
@@ -129,8 +177,51 @@ G_STMT_START {                                                      \
   fail_unless(strcmp (first, second) == 0,                          \
     "'" #a "' (%s) is not equal to '" #b"' (%s)", first, second);   \
 } G_STMT_END;
+/**
+ * assert_equals_string:
+ * @a: a string literal or expression
+ * @b: a string literal or expression
+ *
+ * This macro checks that @a and @b are equal (as per strcmp) and aborts if
+ * this is not the case, printing both expressions and the values they
+ * evaluated to. This macro is for use in unit tests.
+ */
 #define assert_equals_string(a, b) fail_unless_equals_string(a, b)
 
+/**
+ * fail_unless_equals_float:
+ * @a: a #gdouble or #gfloat value or expression
+ * @b: a #gdouble or #gfloat value or expression
+ *
+ * This macro checks that @a and @b are (almost) equal and aborts if this
+ * is not the case, printing both expressions and the values they evaluated
+ * to. This macro is for use in unit tests.
+ *
+ * Since: 0.10.14
+ */
+#define fail_unless_equals_float(a, b)                            \
+G_STMT_START {                                                    \
+  double first = a;                                               \
+  double second = b;                                              \
+  /* This will only work for 'normal' values and values around 0, \
+   * which should be good enough for our purposes here */         \
+  fail_unless(fabs (first - second) < 0.0000001,                  \
+    "'" #a "' (%g) is not equal to '" #b "' (%g)", first, second);\
+} G_STMT_END;
+
+/**
+ * assert_equals_float:
+ * @a: a #gdouble or #gfloat value or expression
+ * @b: a #gdouble or #gfloat value or expression
+ *
+ * This macro checks that @a and @b are (almost) equal and aborts if this
+ * is not the case, printing both expressions and the values they evaluated
+ * to. This macro is for use in unit tests.
+ *
+ * Since: 0.10.14
+ */
+#define assert_equals_float(a, b) fail_unless_equals_float(a, b)
+
 
 /***
  * thread test macros and variables
@@ -289,8 +380,6 @@ fail_unless (gst_element_set_state (element,			\
   state) == ret,						\
   "could not change state to " #state);
 
-#endif /* __GST_CHECK_H__ */
-
 #define GST_CHECK_MAIN(name)					\
 int main (int argc, char **argv)				\
 {								\
@@ -298,3 +387,7 @@ int main (int argc, char **argv)				\
   gst_check_init (&argc, &argv);				\
   return gst_check_run_suite (s, # name, __FILE__);		\
 }
+
+G_END_DECLS
+
+#endif /* __GST_CHECK_H__ */
