@@ -55,17 +55,9 @@
 #include "gstnettimepacket.h"
 #include "gstnetclientclock.h"
 
-GST_DEBUG_CATEGORY (ncc_debug);
+GST_DEBUG_CATEGORY_STATIC (ncc_debug);
 #define GST_CAT_DEFAULT (ncc_debug)
 
-/* #define DEBUGGING_ENABLED */
-
-#ifdef DEBUGGING_ENABLED
-#define DEBUG(x, args...) g_print (x "\n", ##args)
-#else
-#define DEBUG(x, args...)       /* nop */
-#endif
-
 /* the select call is also performed on the control sockets, that way
  * we can send special commands to unblock or restart the select call */
 #define CONTROL_RESTART        'R'      /* restart the select call */
@@ -89,6 +81,10 @@ G_STMT_START {                                  \
 #define DEFAULT_PORT            5637
 #define DEFAULT_TIMEOUT         GST_SECOND
 
+#ifdef G_OS_WIN32
+#define getsockname(sock,addr,len) getsockname(sock,addr,(int *)len)
+#endif
+
 enum
 {
   PROP_0,
@@ -110,6 +106,21 @@ static void gst_net_client_clock_get_property (GObject * object, guint prop_id,
 
 static void gst_net_client_clock_stop (GstNetClientClock * self);
 
+#ifdef G_OS_WIN32
+static int
+inet_aton (const char *c, struct in_addr *paddr)
+{
+  /* note that inet_addr is deprecated on unix because
+   * inet_addr returns -1 (INADDR_NONE) for the valid 255.255.255.255
+   * address. */
+  paddr->s_addr = inet_addr (c);
+  if (paddr->s_addr == INADDR_NONE)
+    return 0;
+
+  return 1;
+}
+#endif
+
 static void
 gst_net_client_clock_base_init (gpointer g_class)
 {
@@ -143,6 +154,18 @@ gst_net_client_clock_init (GstNetClientClock * self,
 {
   GstClock *clock = GST_CLOCK_CAST (self);
 
+#ifdef G_OS_WIN32
+  WSADATA w;
+  int error = WSAStartup (0x0202, &w);
+
+  if (error) {
+    GST_DEBUG_OBJECT (self, "Error on WSAStartup");
+  }
+  if (w.wVersion != 0x0202) {
+    WSACleanup ();
+  }
+#endif
+
   self->port = DEFAULT_PORT;
   self->address = g_strdup (DEFAULT_ADDRESS);
 
@@ -180,6 +203,10 @@ gst_net_client_clock_finalize (GObject * object)
   g_free (self->servaddr);
   self->servaddr = NULL;
 
+#ifdef G_OS_WIN32
+  WSACleanup ();
+#endif
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -257,8 +284,8 @@ gst_net_client_clock_observe_times (GstNetClientClock * self,
 
 bogus_observation:
   {
-    GST_WARNING_OBJECT (self, "time packet receive time < send time (%",
-        GST_TIME_FORMAT, " < %" GST_TIME_FORMAT ")", GST_TIME_ARGS (local_1),
+    GST_WARNING_OBJECT (self, "time packet receive time < send time (%"
+        GST_TIME_FORMAT " < %" GST_TIME_FORMAT ")", GST_TIME_ARGS (local_1),
         GST_TIME_ARGS (local_2));
     return;
   }
@@ -284,8 +311,17 @@ gst_net_client_clock_do_select (GstNetClientClock * self, fd_set * readfds)
       diff = gst_clock_get_internal_time (GST_CLOCK (self));
       GST_TIME_TO_TIMEVAL (self->current_timeout, tv);
 
+#ifdef G_OS_WIN32
+      if (((max_sock + 1) != READ_SOCKET (self)) ||
+          ((max_sock + 1) != WRITE_SOCKET (self))) {
+        ret =
+            select (max_sock + 1, readfds, NULL, NULL, (struct timeval *) ptv);
+      } else {
+        ret = 1;
+      }
+#else
       ret = select (max_sock + 1, readfds, NULL, NULL, (struct timeval *) ptv);
-
+#endif
       diff = gst_clock_get_internal_time (GST_CLOCK (self)) - diff;
 
       if (diff > self->current_timeout)
@@ -340,12 +376,12 @@ gst_net_client_clock_thread (gpointer data)
         int res;
 
         READ_COMMAND (self, command, res);
-        if (res < 0) {
+        if (res <= 0) {
           GST_LOG_OBJECT (self, "no more commands");
           break;
         }
 
-        DEBUG ("control message: '%c'", command);
+        GST_LOG_OBJECT (self, "control message: '%c'", command);
         switch (command) {
           case CONTROL_STOP:
             /* break out of the select loop */
@@ -363,13 +399,13 @@ gst_net_client_clock_thread (gpointer data)
       continue;
     } else if (ret == 0) {
       /* timed out, let's send another packet */
-      DEBUG ("timed out");
+      GST_DEBUG_OBJECT (self, "timed out");
 
       packet = gst_net_time_packet_new (NULL);
 
       packet->local_time = gst_clock_get_internal_time (GST_CLOCK (self));
 
-      DEBUG ("sending packet, local time = %" GST_TIME_FORMAT,
+      GST_DEBUG_OBJECT (self, "sending packet, local time = %" GST_TIME_FORMAT,
           GST_TIME_ARGS (packet->local_time));
       gst_net_time_packet_send (packet, self->sock,
           (struct sockaddr *) self->servaddr, sizeof (struct sockaddr_in));
@@ -390,10 +426,13 @@ gst_net_client_clock_thread (gpointer data)
       if (!packet)
         goto receive_error;
 
-      DEBUG ("got packet back");
-      DEBUG ("local_1 = %" GST_TIME_FORMAT, GST_TIME_ARGS (packet->local_time));
-      DEBUG ("remote = %" GST_TIME_FORMAT, GST_TIME_ARGS (packet->remote_time));
-      DEBUG ("local_2 = %" GST_TIME_FORMAT, GST_TIME_ARGS (new_local));
+      GST_LOG_OBJECT (self, "got packet back");
+      GST_LOG_OBJECT (self, "local_1 = %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (packet->local_time));
+      GST_LOG_OBJECT (self, "remote = %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (packet->remote_time));
+      GST_LOG_OBJECT (self, "local_2 = %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (new_local));
 
       /* observe_times will reset the timeout */
       gst_net_client_clock_observe_times (self, packet->local_time,
@@ -450,12 +489,13 @@ gst_net_client_clock_start (GstNetClientClock * self)
   if (ret < 0)
     goto getsockname_error;
 
-  GST_DEBUG_OBJECT (self, "socket opened on UDP port %hd",
-      ntohs (servaddr.sin_port));
-
   memset (&servaddr, 0, sizeof (servaddr));
   servaddr.sin_family = AF_INET;        /* host byte order */
   servaddr.sin_port = htons (self->port);       /* short, network byte order */
+
+  GST_DEBUG_OBJECT (self, "socket opened on UDP port %hd",
+      ntohs (servaddr.sin_port));
+
   if (!inet_aton (self->address, &servaddr.sin_addr))
     goto bad_address;
 
@@ -568,12 +608,18 @@ gst_net_client_clock_new (gchar * name, const gchar * remote_address,
       g_warning ("unable to set the base time, expect sync problems!");
   }
 
+#ifdef G_OS_WIN32
+  GST_DEBUG_OBJECT (ret, "creating pipe");
+  if ((iret = pipe (CONTROL_SOCKETS (ret))) < 0)
+    goto no_socket_pair;
+#else
   GST_DEBUG_OBJECT (ret, "creating socket pair");
   if ((iret = socketpair (PF_UNIX, SOCK_STREAM, 0, CONTROL_SOCKETS (ret))) < 0)
     goto no_socket_pair;
 
   fcntl (READ_SOCKET (ret), F_SETFL, O_NONBLOCK);
   fcntl (WRITE_SOCKET (ret), F_SETFL, O_NONBLOCK);
+#endif
 
   if (!gst_net_client_clock_start (ret))
     goto failed_start;
