@@ -78,12 +78,6 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
 GST_DEBUG_CATEGORY_STATIC (gst_fd_src_debug);
 #define GST_CAT_DEFAULT gst_fd_src_debug
 
-static const GstElementDetails gst_fd_src_details =
-GST_ELEMENT_DETAILS ("Disk Source",
-    "Source/File",
-    "Synchronous read from a file",
-    "Erik Walthinsen <omega@cse.ogi.edu>");
-
 enum
 {
   PROP_0,
@@ -131,9 +125,12 @@ gst_fd_src_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
+  gst_element_class_set_details_simple (gstelement_class,
+      "Filedescriptor Source",
+      "Source/File",
+      "Read from a file descriptor", "Erik Walthinsen <omega@cse.ogi.edu>");
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&srctemplate));
-  gst_element_class_set_details (gstelement_class, &gst_fd_src_details);
 }
 
 static void
