@@ -63,12 +63,6 @@
 GST_DEBUG_CATEGORY_STATIC (gst_type_find_element_debug);
 #define GST_CAT_DEFAULT gst_type_find_element_debug
 
-static GstElementDetails gst_type_find_element_details =
-GST_ELEMENT_DETAILS ("TypeFind",
-    "Generic",
-    "Finds the media type of a stream",
-    "Benjamin Otte <in7y118@public.uni-hamburg.de>");
-
 /* generic templates */
 GstStaticPadTemplate type_find_element_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
@@ -167,13 +161,17 @@ gst_type_find_element_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
+  gst_element_class_set_details_simple (gstelement_class,
+      "TypeFind",
+      "Generic",
+      "Finds the media type of a stream",
+      "Benjamin Otte <in7y118@public.uni-hamburg.de>");
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&type_find_element_src_template));
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&type_find_element_sink_template));
-  gst_element_class_set_details (gstelement_class,
-      &gst_type_find_element_details);
 }
+
 static void
 gst_type_find_element_class_init (GstTypeFindElementClass * typefind_class)
 {
@@ -466,8 +464,9 @@ gst_type_find_element_handle_event (GstPad * pad, GstEvent * event)
     case MODE_TYPEFIND:
       switch (GST_EVENT_TYPE (event)) {
         case GST_EVENT_EOS:{
-          GstTypeFindProbability prob;
-          GstCaps *caps;
+          GstTypeFindProbability prob = 0;
+          GstCaps *caps = NULL;
+          gboolean found = FALSE;
 
           GST_INFO_OBJECT (typefind, "Got EOS and no type found yet");
 
@@ -476,16 +475,20 @@ gst_type_find_element_handle_event (GstPad * pad, GstEvent * event)
           if (typefind->store) {
             caps = gst_type_find_helper_for_buffer (GST_OBJECT (typefind),
                 typefind->store, &prob);
+
+            if (caps && prob >= typefind->min_probability) {
+              g_signal_emit (typefind, gst_type_find_element_signals[HAVE_TYPE],
+                  0, prob, caps);
+              found = TRUE;
+            }
+            gst_caps_replace (&caps, NULL);
           }
 
-          if (caps && prob >= typefind->min_probability) {
-            g_signal_emit (typefind, gst_type_find_element_signals[HAVE_TYPE],
-                0, prob, caps);
-          } else {
+          if (!found) {
             GST_ELEMENT_ERROR (typefind, STREAM, TYPE_NOT_FOUND,
                 (NULL), (NULL));
           }
-          gst_caps_replace (&caps, NULL);
+
           stop_typefinding (typefind);
           res = gst_pad_event_default (pad, event);
           break;
@@ -615,6 +618,8 @@ gst_type_find_element_chain_do_typefinding (GstTypeFindElement * typefind)
         "probability is %u which is lower than the required minimum of %u",
         caps, probability, typefind->min_probability);
 
+    gst_caps_replace (&caps, NULL);
+
     if (GST_BUFFER_SIZE (typefind->store) >= TYPE_FIND_MAX_SIZE) {
       GST_ELEMENT_ERROR (typefind, STREAM, TYPE_NOT_FOUND, (NULL), (NULL));
       stop_typefinding (typefind);
@@ -631,6 +636,7 @@ gst_type_find_element_chain_do_typefinding (GstTypeFindElement * typefind)
 
   /* .. and send out the accumulated data */
   stop_typefinding (typefind);
+  gst_caps_unref (caps);
   return GST_FLOW_OK;
 }
 
@@ -763,6 +769,7 @@ gst_type_find_element_change_state (GstElement * element,
 
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+    case GST_STATE_CHANGE_READY_TO_NULL:
       gst_caps_replace (&typefind->caps, NULL);
       g_list_foreach (typefind->cached_events,
           (GFunc) gst_mini_object_unref, NULL);
