@@ -31,12 +31,64 @@ GST_START_TEST (test_deserialize_buffer)
 
   g_value_init (&value, GST_TYPE_BUFFER);
   fail_unless (gst_value_deserialize (&value, "1234567890abcdef"));
+  /* does not increase the refcount */
   buf = GST_BUFFER (gst_value_get_mini_object (&value));
+  ASSERT_MINI_OBJECT_REFCOUNT (buf, "buffer", 1);
 
+  /* does not increase the refcount */
+  buf = gst_value_get_buffer (&value);
   ASSERT_MINI_OBJECT_REFCOUNT (buf, "buffer", 1);
 
   /* cleanup */
-  gst_buffer_unref (buf);
+  g_value_unset (&value);
+}
+
+GST_END_TEST;
+
+/* create and serialize a buffer */
+GST_START_TEST (test_serialize_buffer)
+{
+  GValue value = { 0 };
+  GstBuffer *buf;
+  gchar *serialized;
+  static const char *buf_data = "1234567890abcdef";
+  gint len;
+
+  len = strlen (buf_data);
+  buf = gst_buffer_new_and_alloc (len);
+  memcpy (GST_BUFFER_DATA (buf), buf_data, len);
+  ASSERT_MINI_OBJECT_REFCOUNT (buf, "buffer", 1);
+
+  /* and assign buffer to mini object */
+  g_value_init (&value, GST_TYPE_BUFFER);
+  gst_value_take_buffer (&value, buf);
+  ASSERT_MINI_OBJECT_REFCOUNT (buf, "buffer", 1);
+
+  /* now serialize it */
+  serialized = gst_value_serialize (&value);
+  GST_DEBUG ("serialized buffer to %s", serialized);
+  fail_unless (serialized != NULL);
+
+  /* refcount should not change */
+  ASSERT_MINI_OBJECT_REFCOUNT (buf, "buffer", 1);
+
+  /* cleanup */
+  g_free (serialized);
+  g_value_unset (&value);
+
+  /* take NULL buffer */
+  g_value_init (&value, GST_TYPE_BUFFER);
+  GST_DEBUG ("setting NULL buffer");
+  gst_value_take_buffer (&value, NULL);
+
+  /* now serialize it */
+  GST_DEBUG ("serializing NULL buffer");
+  serialized = gst_value_serialize (&value);
+  /* should return NULL */
+  fail_unless (serialized == NULL);
+
+  g_free (serialized);
+  g_value_unset (&value);
 }
 
 GST_END_TEST;
@@ -267,6 +319,70 @@ GST_START_TEST (test_deserialize_guint_failures)
 
 GST_END_TEST;
 
+GST_START_TEST (test_serialize_flags)
+{
+  GValue value = { 0 };
+  gchar *string;
+  GstSeekFlags flags[] = {
+    0,
+    GST_SEEK_FLAG_NONE,
+    GST_SEEK_FLAG_FLUSH,
+    GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_ACCURATE,
+  };
+  const char *results[] = {
+    "GST_SEEK_FLAG_NONE",
+    "GST_SEEK_FLAG_NONE",
+    "GST_SEEK_FLAG_FLUSH",
+    "GST_SEEK_FLAG_FLUSH+GST_SEEK_FLAG_ACCURATE",
+  };
+  int i;
+
+  g_value_init (&value, GST_TYPE_SEEK_FLAGS);
+
+  for (i = 0; i < G_N_ELEMENTS (flags); ++i) {
+    g_value_set_flags (&value, flags[i]);
+    string = gst_value_serialize (&value);
+    fail_if (string == NULL, "could not serialize flags %d", i);
+    fail_unless (strcmp (string, results[i]) == 0,
+        "resulting value is %s, not %s, for flags #%d", string, results[i], i);
+    g_free (string);
+  }
+}
+
+GST_END_TEST;
+
+
+GST_START_TEST (test_deserialize_flags)
+{
+  GValue value = { 0 };
+  const char *strings[] = {
+    "",
+    "0",
+    "GST_SEEK_FLAG_NONE",
+    "GST_SEEK_FLAG_FLUSH",
+    "GST_SEEK_FLAG_FLUSH+GST_SEEK_FLAG_ACCURATE",
+  };
+  GstSeekFlags results[] = {
+    GST_SEEK_FLAG_NONE,
+    GST_SEEK_FLAG_NONE,
+    GST_SEEK_FLAG_NONE,
+    GST_SEEK_FLAG_FLUSH,
+    GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_ACCURATE,
+  };
+  int i;
+
+  g_value_init (&value, GST_TYPE_SEEK_FLAGS);
+
+  for (i = 0; i < G_N_ELEMENTS (strings); ++i) {
+    fail_unless (gst_value_deserialize (&value, strings[i]),
+        "could not deserialize %s (%d)", strings[i], i);
+    fail_unless (g_value_get_flags (&value) == results[i],
+        "resulting value is %d, not %d, for string %s (%d)",
+        g_value_get_flags (&value), results[i], strings[i], i);
+  }
+}
+
+GST_END_TEST;
 
 GST_START_TEST (test_string)
 {
@@ -354,6 +470,7 @@ GST_START_TEST (test_value_compare)
 {
   GValue value1 = { 0 };
   GValue value2 = { 0 };
+  GValue tmp = { 0 };
 
   g_value_init (&value1, G_TYPE_INT);
   g_value_set_int (&value1, 10);
@@ -436,6 +553,104 @@ GST_START_TEST (test_value_compare)
   g_value_unset (&value1);
   g_value_unset (&value2);
 
+  /* Check that lists are equal regardless of order */
+  g_value_init (&value1, GST_TYPE_LIST);
+  g_value_init (&tmp, G_TYPE_INT);
+  g_value_set_int (&tmp, 1);
+  gst_value_list_append_value (&value1, &tmp);
+  g_value_set_int (&tmp, 2);
+  gst_value_list_append_value (&value1, &tmp);
+  g_value_set_int (&tmp, 3);
+  gst_value_list_append_value (&value1, &tmp);
+  g_value_set_int (&tmp, 4);
+  gst_value_list_append_value (&value1, &tmp);
+
+  g_value_init (&value2, GST_TYPE_LIST);
+  g_value_set_int (&tmp, 4);
+  gst_value_list_append_value (&value2, &tmp);
+  g_value_set_int (&tmp, 3);
+  gst_value_list_append_value (&value2, &tmp);
+  g_value_set_int (&tmp, 2);
+  gst_value_list_append_value (&value2, &tmp);
+  g_value_set_int (&tmp, 1);
+  gst_value_list_append_value (&value2, &tmp);
+
+  fail_unless (gst_value_compare (&value1, &value2) == GST_VALUE_EQUAL,
+      "value lists with different order were not equal when they should be");
+  fail_unless (gst_value_compare (&value1, &value1) == GST_VALUE_EQUAL,
+      "value lists with same order were not equal when they should be");
+  fail_unless (gst_value_compare (&value2, &value2) == GST_VALUE_EQUAL,
+      "value lists with same order were not equal when they should be");
+
+  /* Carry over the lists to this next check: */
+  /* Lists with different sizes are unequal */
+  g_value_set_int (&tmp, 1);
+  gst_value_list_append_value (&value2, &tmp);
+
+  fail_if (gst_value_compare (&value1, &value2) == GST_VALUE_EQUAL,
+      "Value lists with different size were equal when they shouldn't be");
+
+  /* Carry over the lists to this next check: */
+  /* Lists with same size but list1 contains one more element not in list2 */
+  g_value_set_int (&tmp, 5);
+  gst_value_list_append_value (&value1, &tmp);
+
+  fail_if (gst_value_compare (&value1, &value2) == GST_VALUE_EQUAL,
+      "Value lists with different elements were equal when they shouldn't be");
+  fail_if (gst_value_compare (&value2, &value1) == GST_VALUE_EQUAL,
+      "Value lists with different elements were equal when they shouldn't be");
+
+  g_value_unset (&value1);
+  g_value_unset (&value2);
+  g_value_unset (&tmp);
+
+  /* Arrays are only equal when in the same order */
+  g_value_init (&value1, GST_TYPE_ARRAY);
+  g_value_init (&tmp, G_TYPE_INT);
+  g_value_set_int (&tmp, 1);
+  gst_value_array_append_value (&value1, &tmp);
+  g_value_set_int (&tmp, 2);
+  gst_value_array_append_value (&value1, &tmp);
+  g_value_set_int (&tmp, 3);
+  gst_value_array_append_value (&value1, &tmp);
+  g_value_set_int (&tmp, 4);
+  gst_value_array_append_value (&value1, &tmp);
+
+  g_value_init (&value2, GST_TYPE_ARRAY);
+  g_value_set_int (&tmp, 4);
+  gst_value_array_append_value (&value2, &tmp);
+  g_value_set_int (&tmp, 3);
+  gst_value_array_append_value (&value2, &tmp);
+  g_value_set_int (&tmp, 2);
+  gst_value_array_append_value (&value2, &tmp);
+  g_value_set_int (&tmp, 1);
+  gst_value_array_append_value (&value2, &tmp);
+
+  fail_if (gst_value_compare (&value1, &value2) == GST_VALUE_EQUAL,
+      "Value arrays with different order were equal when they shouldn't be");
+  fail_unless (gst_value_compare (&value1, &value1) == GST_VALUE_EQUAL,
+      "Identical value arrays were not equal when they should be");
+  fail_unless (gst_value_compare (&value2, &value2) == GST_VALUE_EQUAL,
+      "Identical value arrays were not equal when they should be");
+
+  /* Carry over the arrays to this next check: */
+  /* Arrays with different sizes are unequal */
+  g_value_unset (&value2);
+  g_value_init (&value2, GST_TYPE_ARRAY);
+  g_value_copy (&value1, &value2);
+
+  g_value_set_int (&tmp, 1);
+  gst_value_array_append_value (&value2, &tmp);
+
+  fail_if (gst_value_compare (&value1, &value2) == GST_VALUE_EQUAL,
+      "Value arrays with different size were equal when they shouldn't be");
+  /* order should not matter */
+  fail_if (gst_value_compare (&value2, &value1) == GST_VALUE_EQUAL,
+      "Value arrays with different size were equal when they shouldn't be");
+
+  g_value_unset (&value1);
+  g_value_unset (&value2);
+  g_value_unset (&tmp);
 }
 
 GST_END_TEST;
@@ -1364,6 +1579,45 @@ GST_START_TEST (test_fraction_range)
 
 GST_END_TEST;
 
+GST_START_TEST (test_serialize_deserialize_format_enum)
+{
+  GstStructure *s, *s2;
+  GstFormat foobar_fmt;
+  gchar *str, *str2, *end = NULL;
+
+  /* make sure custom formats are serialised properly as well */
+  foobar_fmt = gst_format_register ("foobar", "GST_FORMAT_FOOBAR");
+  fail_unless (foobar_fmt != GST_FORMAT_UNDEFINED);
+
+  s = gst_structure_new ("foo/bar", "format1", GST_TYPE_FORMAT,
+      GST_FORMAT_BYTES, "format2", GST_TYPE_FORMAT, GST_FORMAT_TIME,
+      "format3", GST_TYPE_FORMAT, GST_FORMAT_DEFAULT, "format4",
+      GST_TYPE_FORMAT, foobar_fmt, NULL);
+
+  str = gst_structure_to_string (s);
+  GST_LOG ("Got structure string '%s'", GST_STR_NULL (str));
+  fail_unless (str != NULL);
+  fail_unless (strstr (str, "TIME") != NULL);
+  fail_unless (strstr (str, "BYTE") != NULL);
+  fail_unless (strstr (str, "DEFAULT") != NULL);
+  fail_unless (strstr (str, "FOOBAR") != NULL);
+
+  s2 = gst_structure_from_string (str, &end);
+  fail_unless (s2 != NULL);
+
+  str2 = gst_structure_to_string (s2);
+  fail_unless (str2 != NULL);
+
+  fail_unless (g_str_equal (str, str2));
+
+  g_free (str);
+  g_free (str2);
+  gst_structure_free (s);
+  gst_structure_free (s2);
+}
+
+GST_END_TEST;
+
 Suite *
 gst_value_suite (void)
 {
@@ -1372,12 +1626,16 @@ gst_value_suite (void)
 
   suite_add_tcase (s, tc_chain);
   tcase_add_test (tc_chain, test_deserialize_buffer);
+  tcase_add_test (tc_chain, test_serialize_buffer);
   tcase_add_test (tc_chain, test_deserialize_gint);
   tcase_add_test (tc_chain, test_deserialize_gint_failures);
   tcase_add_test (tc_chain, test_deserialize_guint);
   tcase_add_test (tc_chain, test_deserialize_guint_failures);
   tcase_add_test (tc_chain, test_deserialize_gint64);
   tcase_add_test (tc_chain, test_deserialize_gstfraction);
+  tcase_add_test (tc_chain, test_serialize_flags);
+  tcase_add_test (tc_chain, test_deserialize_flags);
+  tcase_add_test (tc_chain, test_serialize_deserialize_format_enum);
   tcase_add_test (tc_chain, test_string);
   tcase_add_test (tc_chain, test_deserialize_string);
   tcase_add_test (tc_chain, test_value_compare);
@@ -1392,19 +1650,4 @@ gst_value_suite (void)
   return s;
 }
 
-int
-main (int argc, char **argv)
-{
-  int nf;
-
-  Suite *s = gst_value_suite ();
-  SRunner *sr = srunner_create (s);
-
-  gst_check_init (&argc, &argv);
-
-  srunner_run_all (sr, CK_NORMAL);
-  nf = srunner_ntests_failed (sr);
-  srunner_free (sr);
-
-  return nf;
-}
+GST_CHECK_MAIN (gst_value);
