@@ -430,6 +430,20 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
           gst_tag_list_free (tags);
         }
         break;
+      case GST_MESSAGE_INFO:{
+        GError *gerror;
+        gchar *debug;
+        gchar *name = gst_object_get_path_string (GST_MESSAGE_SRC (message));
+
+        gst_message_parse_info (message, &gerror, &debug);
+        if (debug) {
+          g_print (_("INFO:\n%s\n"), debug);
+        }
+        g_error_free (gerror);
+        g_free (debug);
+        g_free (name);
+        break;
+      }
       case GST_MESSAGE_WARNING:{
         GError *gerror;
         gchar *debug;
@@ -468,8 +482,11 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
 
         /* ignore when we are buffering since then we mess with the states
          * ourselves. */
-        if (buffering)
+        if (buffering) {
+          fprintf (stderr,
+              _("Prerolled, waiting for buffering to finish...\n"));
           break;
+        }
 
         /* if we reached the final target state, exit */
         if (target_state == GST_STATE_PAUSED && new == target_state)
@@ -482,14 +499,15 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
         gint percent;
 
         gst_message_parse_buffering (message, &percent);
-        fprintf (stderr, "buffering... %d\r", percent);
+        fprintf (stderr, "buffering... %d  \r", percent);
 
         if (percent == 100) {
           /* a 100% message means buffering is done */
           buffering = FALSE;
           /* if the desired state is playing, go back */
           if (target_state == GST_STATE_PLAYING) {
-            fprintf (stderr, _("Setting pipeline to PLAYING ...\n"));
+            fprintf (stderr,
+                _("Done buffering, setting pipeline to PLAYING ...\n"));
             gst_element_set_state (pipeline, GST_STATE_PLAYING);
           } else
             goto exit;
@@ -497,10 +515,10 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
           /* buffering busy */
           if (buffering == FALSE && target_state == GST_STATE_PLAYING) {
             /* we were not buffering but PLAYING, PAUSE  the pipeline. */
-            fprintf (stderr, _("Setting pipeline to PAUSED ...\n"));
+            fprintf (stderr, _("Buffering, setting pipeline to PAUSED ...\n"));
             gst_element_set_state (pipeline, GST_STATE_PAUSED);
-            buffering = TRUE;
           }
+          buffering = TRUE;
         }
         break;
       }
@@ -540,8 +558,6 @@ exit:
 int
 main (int argc, char *argv[])
 {
-  gint i, j;
-
   /* options */
   gboolean verbose = FALSE;
   gboolean no_fault = FALSE;
@@ -601,25 +617,6 @@ main (int argc, char *argv[])
 
   gst_tools_print_version ("gst-launch");
 
-  /* FIXpopt: strip short args, too. We do it ourselves for now */
-  j = 1;
-  for (i = 1; i < argc; i++) {
-    if (*(argv[i]) == '-') {
-      if (strlen (argv[i]) == 2) {
-        gchar *c = argv[i];
-
-        c++;
-        if (*c == 'X' || *c == 'o') {
-          i++;
-        }
-      }
-    } else {
-      argv[j] = argv[i];
-      j++;
-    }
-  }
-  argc = j;
-
 #ifndef DISABLE_FAULT_HANDLER
   if (!no_fault)
     fault_setup ();
@@ -748,7 +745,8 @@ main (int argc, char *argv[])
 
     fprintf (stderr, _("Setting pipeline to PAUSED ...\n"));
     gst_element_set_state (pipeline, GST_STATE_PAUSED);
-    gst_element_get_state (pipeline, &state, &pending, GST_CLOCK_TIME_NONE);
+    if (!caught_error)
+      gst_element_get_state (pipeline, &state, &pending, GST_CLOCK_TIME_NONE);
     fprintf (stderr, _("Setting pipeline to READY ...\n"));
     gst_element_set_state (pipeline, GST_STATE_READY);
     gst_element_get_state (pipeline, &state, &pending, GST_CLOCK_TIME_NONE);
