@@ -27,86 +27,24 @@
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
-#include <popt.h>
 #include <glib.h>
 
-enum
-{
-  ARG_MM = 1,
-  ARG_LIST_MM,
-  ARG_PRINT,
-  ARG_HELP
-};
-
 /* global statics for option parsing */
 static gboolean _print = FALSE;
 static gchar *_arg_mm = NULL;
 static gboolean _arg_list_mm = FALSE;
 
-/* callback to parse arguments */
-static void
-popt_callback (poptContext context, enum poptCallbackReason reason,
-    const struct poptOption *option, const char *arg, void *data)
-{
-  if (reason == POPT_CALLBACK_REASON_OPTION) {
-    switch (option->val) {
-      case ARG_MM:
-        _arg_mm = g_strdup (arg);
-        break;
-      case ARG_LIST_MM:
-        _arg_list_mm = TRUE;
-        break;
-      case ARG_PRINT:
-        _print = TRUE;
-        break;
-      case ARG_HELP:
-        poptPrintHelp (context, stdout, 0);
-        g_print ("\n");
-        break;
-    }
-  } else {
-    g_print ("Unknown reason for callback\n");
-  }
-}
-
 /* popt options table for the wrapper */
-static struct poptOption wrapper_options[] = {
-  {NULL, '\0',
-        POPT_ARG_CALLBACK,
-      (void *) &popt_callback, 0, NULL, NULL},
-  {"help", '\0',
-        POPT_ARG_NONE | POPT_ARGFLAG_DOC_HIDDEN,
-      NULL, ARG_HELP, ("Show help"), NULL},
-  {"?", '\0',
-        POPT_ARG_NONE | POPT_ARGFLAG_STRIP | POPT_ARGFLAG_ONEDASH
-        | POPT_ARGFLAG_DOC_HIDDEN,
-      NULL, ARG_HELP, NULL, NULL},
-  /* We cheat by specifying -p as long "p" with onedash, so that it
-     also gets stripped properly from our arg flags */
-  {"p", '\0',
-        POPT_ARG_NONE | POPT_ARGFLAG_STRIP | POPT_ARGFLAG_ONEDASH
-        | POPT_ARGFLAG_DOC_HIDDEN,
-      NULL, ARG_PRINT, NULL, NULL},
-  {"print", '\0',
-        POPT_ARG_NONE | POPT_ARGFLAG_STRIP,
-      NULL, ARG_PRINT, ("Print wrapped command line"), NULL},
-  {"gst-mm", '\0',
-        POPT_ARG_STRING | POPT_ARGFLAG_STRIP,
-      NULL, ARG_MM, ("Force major/minor version"), NULL},
-  {"gst-list-mm", '\0',
-        POPT_ARG_NONE | POPT_ARGFLAG_STRIP,
-      NULL, ARG_LIST_MM, ("List found major/minor versions"), NULL},
-  POPT_TABLEEND
+static GOptionEntry wrapper_options[] = {
+  {"print", 'p', 0, G_OPTION_ARG_NONE, &_print,
+      "print wrapped command line options", NULL},
+  {"gst-mm", 0, 0, G_OPTION_ARG_STRING, &_arg_mm,
+      "Force major/minor version", "VERSION"},
+  {"gst-list-mm", 0, 0, G_OPTION_ARG_NONE, &_arg_list_mm,
+      "List found major/minor versions", NULL},
+  {NULL}
 };
 
-/* helper table including our wrapper options */
-static struct poptOption options[] = {
-  {NULL, '\0', POPT_ARG_INCLUDE_TABLE, wrapper_options, 0,
-      "Wrapper options:", NULL},
-  POPT_TABLEEND
-};
-
-
 /* print out the major/minor, which is the hash key */
 static void
 hash_print_key (gchar * key, gchar * value)
@@ -114,6 +52,57 @@ hash_print_key (gchar * key, gchar * value)
   g_print ("%s\n", (gchar *) key);
 }
 
+/* return value like strcmp, but compares major/minor numerically */
+static gint
+compare_major_minor (const gchar * first, const gchar * second)
+{
+  gchar **firsts, **seconds;
+  gint fmaj, fmin, smaj, smin;
+  gint ret = 0;
+
+  firsts = g_strsplit (first, ".", 0);
+  seconds = g_strsplit (second, ".", 0);
+
+  if (firsts[0] == NULL || firsts[1] == NULL) {
+    ret = -1;
+    goto beach;
+  }
+  if (seconds[0] == NULL || seconds[1] == NULL) {
+    ret = 1;
+    goto beach;
+  }
+
+  fmaj = atoi (firsts[0]);
+  fmin = atoi (firsts[1]);
+  smaj = atoi (seconds[0]);
+  smin = atoi (seconds[1]);
+
+  if (fmaj < smaj) {
+    ret = -1;
+    goto beach;
+  }
+  if (fmaj > smaj) {
+    ret = 1;
+    goto beach;
+  }
+
+  /* fmaj == smaj */
+  if (fmin < smin) {
+    ret = -1;
+    goto beach;
+  }
+  if (fmin > smin) {
+    ret = 1;
+    goto beach;
+  }
+  ret = 0;
+
+beach:
+  g_strfreev (firsts);
+  g_strfreev (seconds);
+  return ret;
+}
+
 static void
 find_highest_version (gchar * key, gchar * value, gchar ** highest)
 {
@@ -121,7 +110,7 @@ find_highest_version (gchar * key, gchar * value, gchar ** highest)
     /* first value, so just set it */
     *highest = key;
   }
-  if (strcmp (key, *highest) > 0)
+  if (compare_major_minor (key, *highest) > 0)
     *highest = key;
 }
 
@@ -210,13 +199,10 @@ get_candidates (const gchar * dir, const gchar * base)
   GError *error = NULL;
   const gchar *entry;
   gchar *path;
-  gchar *suffix;
+  gchar *suffix, *copy;
 
   gchar *pattern;
-  GPatternSpec *spec;
-
-
-  gchar *test;
+  GPatternSpec *spec, *specexe;
 
   gchar **dirs;
   gchar **cur;
@@ -225,10 +211,13 @@ get_candidates (const gchar * dir, const gchar * base)
 
   candidates = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
-  /* compile our pattern spec */
+  /* compile our pattern specs */
   pattern = g_strdup_printf ("%s-*.*", base);
   spec = g_pattern_spec_new (pattern);
   g_free (pattern);
+  pattern = g_strdup_printf ("%s-*.*.exe", base);
+  specexe = g_pattern_spec_new (pattern);
+  g_free (pattern);
 
   /* get all dirs from the path and prepend with given dir */
   path = g_strdup_printf ("%s%c%s",
@@ -243,8 +232,10 @@ get_candidates (const gchar * dir, const gchar * base)
 
   while (cur != &dirs[0]) {
     --cur;
-    if (!g_file_test (*cur, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))
+    if (!g_file_test (*cur, G_FILE_TEST_EXISTS) ||
+        !g_file_test (*cur, G_FILE_TEST_IS_DIR)) {
       continue;
+    }
 
     gdir = g_dir_open (*cur, 0, &error);
     if (!gdir) {
@@ -253,7 +244,8 @@ get_candidates (const gchar * dir, const gchar * base)
       return NULL;
     }
     while ((entry = g_dir_read_name (gdir))) {
-      if (g_pattern_match_string (spec, entry)) {
+      if (g_pattern_match_string (spec, entry)
+          || g_pattern_match_string (specexe, entry)) {
         gchar *full;
 
         /* is it executable ? */
@@ -266,15 +258,20 @@ get_candidates (const gchar * dir, const gchar * base)
 
         /* strip base and dash from it */
         suffix = g_strdup (&(entry[strlen (base) + 1]));
+        copy = g_strdup (suffix);
+
+        /* strip possible .exe from copy */
+        if (g_strrstr (copy, ".exe"))
+          g_strrstr (copy, ".exe")[0] = '\0';
 
         /* stricter pattern check: check if it only contains digits or dots */
-        test = g_strdup (suffix);
-        g_strcanon (test, "0123456789.", 'X');
-        if (strstr (test, "X")) {
-          g_free (test);
+        g_strcanon (copy, "0123456789.", 'X');
+        if (strstr (copy, "X")) {
+          g_free (suffix);
+          g_free (copy);
           continue;
         }
-        g_free (test);
+        g_free (copy);
         g_hash_table_insert (candidates, suffix, g_strdup (*cur));
       }
     }
@@ -294,23 +291,26 @@ main (int argc, char **argv)
   gchar *highest = NULL;
   gchar *binary;                /* actual binary we're going to run */
   gchar *path = NULL;           /* and its path */
-  poptContext ctx;
-  int nextopt;
-
-  /* parse command line options */
-  ctx = poptGetContext ("gst-run", argc, (const char **) argv, options, 0);
-  poptReadDefaultConfig (ctx, TRUE);
-  while ((nextopt = poptGetNextOpt (ctx)) > 0)
-    /* keep looping to parse */ ;
-
-  argc = poptStrippedArgv (ctx, argc, argv);
-  argv[argc] = NULL;
-  poptFreeContext (ctx);
+  gchar *desc;
+  GOptionContext *ctx;
+  GError *err = NULL;
 
   /* detect stuff */
   dir = get_dir_of_binary (argv[0]);
   base = g_path_get_basename (argv[0]);
 
+  /* parse command line options */
+  desc = g_strdup_printf ("wrapper to call versioned %s", base);
+  ctx = g_option_context_new (desc);
+  g_free (desc);
+  g_option_context_set_ignore_unknown_options (ctx, TRUE);
+  g_option_context_add_main_entries (ctx, wrapper_options, GETTEXT_PACKAGE);
+  if (!g_option_context_parse (ctx, &argc, &argv, &err)) {
+    g_print ("Error initializing: %s\n", err->message);
+    exit (1);
+  }
+  g_option_context_free (ctx);
+
   /* unmangle libtool if necessary */
   unmangle_libtool (&dir, &base);
 
@@ -330,9 +330,13 @@ main (int argc, char **argv)
     highest = NULL;
 
     /* otherwise, just look up the highest version */
-    g_hash_table_foreach (candidates, (GHFunc) find_highest_version, &highest);
+    if (candidates) {
+      g_hash_table_foreach (candidates, (GHFunc) find_highest_version,
+          &highest);
+    }
+
     if (highest == NULL) {
-      g_print ("ERROR: No version of tool %s not found.\n", base);
+      g_print ("ERROR: No version of tool %s found.\n", base);
       return 1;
     }
     dir = g_hash_table_lookup (candidates, highest);
